#!/usr/bin/env python3
"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.



Egeria Coco Pharmaceutical demonstration labs.

This script creates and configures the cocoMDS2 - Governance metadata server


"""


import argparse
import json
from datetime import datetime

from globals import (
    adminUserId,
    cocoCohort,
    cocoMDS2Name,
    cocoMDS5Name,
    cocoMDS6Name,
    corePlatformURL,
    devCohort,
    iotCohort,
    max_paging_size,
)

from pyegeria import CoreServerConfig, Platform, print_exception_response


def config_coco_core(url: str, userid: str):
    disable_ssl_warnings = True
    print("Configuring and activating the Datalake")
    platform_url = url
    admin_user = userid

    # event_bus_config = {
    #     "producer": {
    #         "bootstrap.servers": "{{kafkaEndpoint}}"
    #     },
    #     "consumer": {
    #         "bootstrap.servers": "{{kafkaEndpoint}}"
    #     }
    # }
    event_bus_config = {
        "producer": {"bootstrap.servers": "host.docker.internal:9192"},
        "consumer": {"bootstrap.servers": "host.docker.internal:9192"},
    }

    security_connection_body = {
        "class": "Connection",
        "connectorType": {
            "class": "ConnectorType",
            "connectorProviderClassName": "org.odpi.openmetadata.metadatasecurity.samples.CocoPharmaServerSecurityProvider",
        },
    }

    #
    # Configure MDS2
    #
    mdr_server = cocoMDS2Name
    mdr_server_user_id = "cocoMDS2npa"
    mdr_server_password = "cocoMDS2passw0rd"
    metadataCollectionId = f"{mdr_server}-e915f2fa-aa3g-4396-8bde-bcd65e642b1d"
    metadataCollectionName = "Governance Catalog"

    try:
        o_client = CoreServerConfig(mdr_server, platform_url, admin_user)

        o_client.set_basic_server_properties(
            "Governance Server",
            "Coco Pharmaceuticals",
            platform_url,
            mdr_server_user_id,
            mdr_server_password,
            max_paging_size,
        )

        o_client.set_event_bus(event_bus_config)
        o_client.set_server_security_connection(security_connection_body)
        o_client.add_default_log_destinations()

        # o_client.set_in_mem_local_repository()
        o_client.set_xtdb_local_kv_repository()

        o_client.set_local_metadata_collection_id(metadataCollectionId)
        o_client.set_local_metadata_collection_name(metadataCollectionName)

        o_client.add_cohort_registration(cocoCohort)
        o_client.add_cohort_registration(devCohort)
        o_client.add_cohort_registration(iotCohort)

        # o_client.configure_access_service("asset-catalog", {})
        o_client.configure_access_service("asset-consumer", {})

        o_client.configure_access_service("asset-owner", {})
        o_client.configure_access_service(
            "community-profile", {"KarmaPointPlateau": "500"}
        )
        # o_client.configure_access_service("glossary-view", {})
        # o_client.configure_access_service("subject-area", {})
        o_client.configure_access_service("governance-engine", {})
        o_client.configure_access_service("governance-server", {})
        o_client.configure_access_service("governance-program", {})
        # o_client.configure_access_service("data-privacy", {})
        o_client.configure_access_service("digital-architecture", {})
        o_client.configure_access_service("security-manager", {})
        o_client.configure_access_service("asset-lineage", {})
        o_client.configure_access_service("it-infrastructure", {})
        o_client.configure_access_service("project-management", {})
        print(f"Activating {mdr_server}")
        p_client = Platform(mdr_server, platform_url, admin_user)
        p_client.activate_server_stored_config()
        print(f"{mdr_server} activated")
    except Exception as e:
        print_exception_response(e)
    #
    # Configure MDS5
    #
    disable_ssl_warnings = True

    mdr_server = cocoMDS5Name
    mdr_server_user_id = "cocoMDS5npa"
    mdr_server_password = "cocoMDS5passw0rd"
    metadataCollectionId = f"{mdr_server}-e915f2fa-aa3g-4396-8bde-bcd65e642b1d"
    metadataCollectionName = "Business Systems Catalog"

    print("Configuring " + mdr_server + "...")
    try:
        o_client = CoreServerConfig(mdr_server, platform_url, admin_user)

        o_client.set_basic_server_properties(
            "Business Systems",
            "Coco Pharmaceuticals",
            platform_url,
            mdr_server_user_id,
            mdr_server_password,
            max_paging_size,
        )

        o_client.set_event_bus(event_bus_config)
        o_client.set_server_security_connection(security_connection_body)
        o_client.add_default_log_destinations()

        # o_client.set_in_mem_local_repository()
        o_client.set_xtdb_local_kv_repository()

        o_client.set_local_metadata_collection_id(metadataCollectionId)
        o_client.set_local_metadata_collection_name(metadataCollectionName)

        o_client.add_cohort_registration(cocoCohort)

        proxy_details = (
            "org.odpi.openmetadata.adapters.repositoryservices.readonly.repositoryconnector."
            + "ReadOnlyOMRSRepositoryConnectorProvider"
        )
        o_client.set_repository_proxy_details(proxy_details)
        print(f"Activating {mdr_server}")
        p_client = Platform(mdr_server, platform_url, admin_user)
        p_client.activate_server_stored_config()
        print(f"{mdr_server} activated")
    except Exception as e:
        print_exception_response(e)

    #
    # Configure MDS6
    #
    disable_ssl_warnings = True

    mdr_server = cocoMDS6Name
    platform_url = corePlatformURL
    admin_user = "garygeeke"
    mdr_server_user_id = "cocoMDS6npa"
    mdr_server_password = "cocoMDS6passw0rd"
    metadataCollectionId = f"{mdr_server}-e915f2fa-aa3g-4396-8bde-bcd65e642b1d"
    metadataCollectionName = "Manufacturing Catalog"

    print("Configuring " + mdr_server + "...")

    try:
        o_client = CoreServerConfig(mdr_server, platform_url, admin_user)

        o_client.set_basic_server_properties(
            "Manufacturing",
            "Coco Pharmaceuticals",
            platform_url,
            mdr_server_user_id,
            mdr_server_password,
            max_paging_size,
        )

        o_client.set_event_bus(event_bus_config)

        o_client.set_server_security_connection(security_connection_body)
        o_client.add_default_log_destinations()

        # o_client.set_in_mem_local_repository()
        o_client.set_xtdb_local_kv_repository()

        o_client.set_local_metadata_collection_id(metadataCollectionId)
        o_client.set_local_metadata_collection_name(metadataCollectionName)

        o_client.add_cohort_registration(cocoCohort)
        o_client.add_cohort_registration(iotCohort)

        access_service_options = {
            "SupportedZones": ["manufacturing"],
            "DefaultZones": ["manufacturing"],
        }

        # o_client.configure_access_service("asset-catalog", access_service_options)
        o_client.configure_access_service("asset-consumer", access_service_options)
        o_client.configure_access_service("asset-owner", access_service_options)
        o_client.configure_access_service(
            "community-profile", {"KarmaPointPlateau": "500"}
        )
        # o_client.configure_access_service("glossary-view", {})
        o_client.configure_access_service("data-science", access_service_options)
        # o_client.configure_access_service("subject-area", {})
        o_client.configure_access_service("asset-manager", access_service_options)
        o_client.configure_access_service("governance-engine", access_service_options)
        o_client.configure_access_service("governance-server", access_service_options)
        o_client.configure_access_service("asset-owner", access_service_options)
        # o_client.configure_access_service("data-engine", access_service_options)
        o_client.configure_access_service("data-manager", access_service_options)
        o_client.configure_access_service("it-infrastructure", access_service_options)
        o_client.configure_access_service("project-management", access_service_options)

        print(f"Activating {mdr_server}")
        p_client = Platform(mdr_server, platform_url, admin_user)
        p_client.activate_server_stored_config()
        print(f"{mdr_server} activated")
    except Exception as e:
        print_exception_response(e)


def main():
    parser = argparse.ArgumentParser()

    parser.add_argument("--url", help="URL Platform to connect to")
    parser.add_argument("--userid", help="User Id")
    args = parser.parse_args()

    url = args.url if args.url is not None else corePlatformURL
    userid = args.userid if args.userid is not None else adminUserId

    config_coco_core(url, userid)


if __name__ == "__main__":
    main()



"""
This file contains glossary-related object_action functions for processing Egeria Markdown
"""

import json
import os
import re
from typing import List, Optional
from loguru import logger
from rich import print
from rich.console import Console
from rich.markdown import Markdown

from md_processing.md_processing_utils.common_md_proc_utils import (parse_upsert_command, parse_view_command,
                                                                    sync_collection_memberships)
from md_processing.md_processing_utils.common_md_utils import update_element_dictionary, set_update_body, \
    set_element_status_request_body, set_element_prop_body, set_delete_request_body, set_rel_request_body, set_peer_gov_def_request_body, \
    set_rel_request_body, set_create_body, set_object_classifications, set_product_body, set_rel_prop_body

from md_processing.md_processing_utils.extraction_utils import (extract_command_plus, update_a_command)
from md_processing.md_processing_utils.md_processing_constants import (load_commands)
from pyegeria import DEBUG_LEVEL, body_slimmer, to_pascal_case, PyegeriaException, print_basic_exception, print_exception_table
from pyegeria.egeria_tech_client import EgeriaTech



from md_processing.md_processing_utils.common_md_utils import (debug_level, print_msg, set_debug_level,
                                                               get_element_dictionary, update_element_dictionary,
                                                               )
from md_processing.md_processing_utils.extraction_utils import (extract_command_plus, extract_command,
                                                                process_simple_attribute, process_element_identifiers,
                                                                update_a_command, extract_attribute,
                                                                get_element_by_name, process_name_list)

from pyegeria._globals import (NO_GLOSSARIES_FOUND, NO_ELEMENTS_FOUND, NO_CATEGORIES_FOUND)
from pyegeria.egeria_tech_client import EgeriaTech
from pyegeria.utils import make_format_set_name_from_type, body_slimmer


# EGERIA_WIDTH = int(os.environ.get("EGERIA_WIDTH", "170"))
# console = Console(width=EGERIA_WIDTH)

def update_term_categories(egeria_client: EgeriaTech, term_guid: str, categories_exist: bool,
                           categories_list: List[str]) -> None:
    """

    Adds or removes a term to/from specified categories in a glossary.

    This function associates a term, identified by its GUID, with one or more
    categories. It uses the provided EgeriaTech client to assign the term
    to the categories. If the GUID of a category is not readily available, it
    is retrieved either from a pre-loaded dictionary or through a client lookup.

    Args:
        egeria_client (EgeriaTech): The client to interact with the glossary.
        term_guid (str): The GUID of the term to be associated with categories.
        categories_exist (bool): Flag indicating whether the categories already
                                 exist.
        categories_list (List[str]): A list of category names to associate with
                                     the term.

    Returns:
        None
    """
    to_be_cat_guids: list[str] = []
    # find the categories a term is currently in.
    existing_categories = egeria_client.get_categories_for_term(term_guid)
    if type(existing_categories) is str:
        current_categories = []
    else:
        current_categories = [cat['elementHeader']['guid'] for cat in existing_categories]

    if categories_exist is True and categories_list is not None:
        if type(categories_list) is str:
            categories_list = categories_list.split(",").trim()
        for category in categories_list:
            cat_guid = None
            cat_el = category.strip()
            element_dict = get_element_dictionary()
            if cat_el in element_dict:
                cat = element_dict.get(cat_el, None)
                cat_guid = cat.get('guid', None) if cat else None
            if cat_guid is None:
                cat_guid = egeria_client.__get_guid__(qualified_name=cat_el)
                update_element_dictionary(cat_el, {'guid': cat_guid})
            to_be_cat_guids.append(cat_guid)

        for cat in to_be_cat_guids:
            if cat not in current_categories:
                egeria_client.add_term_to_category(term_guid, cat)
                current_categories.append(cat)
                msg = f"Added term {term_guid} to category {cat}"
                print_msg("DEBUG-INFO", msg, debug_level)

        for cat in current_categories:
            if cat not in to_be_cat_guids:
                egeria_client.remove_term_from_category(term_guid, cat)
                msg = f"Removed term {term_guid} from category {cat}"
                print_msg("DEBUG-INFO", msg, debug_level)
    else:  # No categories specified - so remove any categories a term is in
        for cat in current_categories:
            egeria_client.remove_term_from_category(term_guid, cat)
            msg = f"Removed term {term_guid} from category {cat}"
            print_msg("DEBUG-INFO", msg, debug_level)


def process_glossary_upsert_command(egeria_client: EgeriaTech, txt: str, directive: str = "display") -> Optional[str]:
    """
    Processes a glossary create or update object_action by extracting key attributes such as
    glossary name, language, description, and usage from the given text.

    :param txt: A string representing the input cell to be processed for
        extracting glossary-related attributes.
    :param directive: an optional string indicating the directive to be used - display, validate or execute
    :return: A string summarizing the outcome of the processing.
    """

    command, object_type, object_action = extract_command_plus(txt)
    print(Markdown(f"# {command}\n"))

    parsed_output = parse_upsert_command(egeria_client, object_type, object_action, txt, directive)
    if not parsed_output:
        logger.error(f"No output for `{object_action}`")
        return None

    valid = parsed_output['valid']
    exists = parsed_output['exists']

    qualified_name = parsed_output.get('qualified_name', None)
    guid = parsed_output.get('guid', None)

    print(Markdown(parsed_output['display']))

    logger.debug(json.dumps(parsed_output, indent=4))

    attributes = parsed_output['attributes']

    display_name = attributes['Display Name'].get('value', None)
    status = attributes.get('Status', {}).get('value', None)
    #

    if directive == "display":
        return None
    elif directive == "validate":
        if valid:
            print(Markdown(f"==> Validation of {command} completed successfully!\n"))
        else:
            msg = f"Validation failed for object_action `{command}`\n"
        return valid

    elif directive == "process":
        try:
            obj = "Glossary"
            #   Set the property body for a glossary collection
            #
            prop_body = set_element_prop_body(obj, qualified_name, attributes)
            prop_body["languager"] = attributes.get('Language', {}).get('value', None)
            prop_body["usage"] = attributes.get('Usage', {}).get('value', None)

            if object_action == "Update":
                if not exists:
                    msg = (f" Element `{display_name}` does not exist! Updating result document with Create "
                           f"{object_action}\n")
                    logger.error(msg)
                    return update_a_command(txt, object_action, object_type, qualified_name, guid)
                elif not valid:
                    return None
                else:
                    print(Markdown(
                        f"==> Validation of {command} completed successfully! Proceeding to apply the changes.\n"))


                body = set_update_body(obj, attributes)
                body['properties'] = prop_body

                egeria_client.update_collection(guid, body)
                if status:
                    egeria_client.update_collection_status(guid, status)

                logger.success(f"Updated  {object_type} `{display_name}` with GUID {guid}\n\n___")
                update_element_dictionary(qualified_name, {
                    'guid': guid, 'display_name': display_name
                    })
                return egeria_client.get_collection_by_guid(guid, element_type='Glossary',
                                                            output_format='MD', report_spec = "DrE-Glossary")


            elif object_action == "Create":
                if valid is False and exists:
                    msg = (f"  Digital Product `{display_name}` already exists and result document updated changing "
                           f"`Create` to `Update` in processed output\n\n___")
                    logger.error(msg)
                    return update_a_command(txt, object_action, object_type, qualified_name, guid)

                else:
                    body = set_create_body(object_type,attributes)

                    # if this is a root or folder (maybe more in the future), then make sure that the classification is set.
                    body["initialClassifications"] = set_object_classifications(object_type, attributes, ["Taxonomy", "CanonicalVocabulary"])
                    if object_type == "Taxonomy":
                        body["initialClassifications"]['Taxonomy']['organizingPrinciple'] = attributes.get('Organizing Principle', {}).get('value', None)
                    elif object_type == "CanonicalVocabulary":
                        body["initialClassifications"]['CanonicalVocabulary']['usage'] = attributes.get('Usage', {}).get('value', None)

                    body["properties"] = prop_body

                    guid = egeria_client.create_collection(body = body)
                    if guid:
                        update_element_dictionary(qualified_name, {
                            'guid': guid, 'display_name': display_name
                            })
                        msg = f"Created Element `{display_name}` with GUID {guid}\n\n___"
                        logger.success(msg)
                        return egeria_client.get_collection_by_guid(guid, obj, output_format='MD')
                    else:
                        msg = f"Failed to create element `{display_name}` with GUID {guid}\n\n___"
                        logger.error(msg)
                        return None

        except PyegeriaException as e:
            logger.error(f"Pyegeria error performing {command}: {e}")
            print_basic_exception(e)
            return None
        except Exception as e:
            logger.error(f"Error performing {command}: {e}")
    else:
        return None


def process_category_upsert_command(egeria_client: EgeriaTech, txt: str, directive: str = "display") -> Optional[str]:
    pass
#     """
#     Processes a glossary category create or update object_action by extracting key attributes such as
#     category name, qualified, description, and anchor glossary from the given txt..
#
#     :param txt: A string representing the input cell to be processed for
#         extracting category-related attributes.
#     :param directive: an optional string indicating the directive to be used - display, validate or execute
#     :return: A string summarizing the outcome of the processing.
#     """
#     valid = True
#     set_debug_level(directive)
#
#     command, object_type, object_action = extract_command_plus(txt)
#     print(Markdown(f"# {command}\n"))
#
#     category_name = process_simple_attribute(txt, ['Category Name', 'category_name', 'Cat'])
#     print(Markdown(f"{pre_command} `{command}` for category: `\'{category_name}\'` with directive: `{directive}` "))
#
#     owning_glossary_name = extract_attribute(txt, ['Owning Glossary', 'In Glossary'])
#     description = process_simple_attribute(txt, ['Description'])
#     q_name = process_simple_attribute(txt, ['Qualified Name'])
#
#     parent_category_name = process_simple_attribute(txt, PARENT_CATEGORY_LABELS, "INFO")
#
#     element_labels = CATEGORY_NAME_LABELS
#     element_labels.append('Display Name')
#     # Check if category exists (and get qname and guid)
#     if category_name is None:
#         valid = False
#         known_q_name, known_guid, category_exists = None
#     else:
#         element_labels = CATEGORY_NAME_LABELS
#         element_labels.append('Display Name')
#         known_q_name, known_guid, valid, category_exists = process_element_identifiers(egeria_client, object_type,
#                                                                                        element_labels, txt,
#                                                                                        object_action, None)
#
#     # Check if owning glossary exists (and get qname)
#     if owning_glossary_name is None:
#         valid = False
#         known_glossary_q_name, known_glossary__guid, glossary_exists = None
#
#     else:
#         known_glossary_q_name, known_glossary_guid, valid, owning_glossary_exists = process_element_identifiers(
#             egeria_client, "Glossary", GLOSSARY_NAME_LABELS, txt, EXISTS_REQUIRED, None)
#
#     if parent_category_name:
#         _, parent_guid, parent_valid, parent_exists = get_element_by_name(egeria_client, 'Glossary Categories',
#                                                                           parent_category_name)
#     else:
#         parent_guid = None
#         parent_exists = False
#         parent_valid = False
#
#     category_display = (
#         f"\n* Command: {command}\n\t* Category: {category_name}\n\t* In Glossary: {owning_glossary_name}\n\t"
#         f"* Description:\n{description}\n\t* Parent Category: {parent_category_name}\n\t"
#         f"* Qualified Name: {q_name}\n\t")
#
#     if object_action == 'Update':
#         guid = process_simple_attribute(txt, ['GUID', 'guid', 'Guid'])
#
#         category_display += (f"* GUID: {guid}\n\n")
#         if not category_exists:
#             msg = f"Category {category_name} can't be updated; {category_name} not found."
#             print_msg(ERROR, msg, debug_level)
#             valid = False
#         else:
#             msg = f"Glossary can be updated; {category_name} found"
#             print_msg(ALWAYS, msg, debug_level)
#
#     elif object_action == "Create":
#         if category_exists:
#             msg = f"Category {category_name} can't be created because it already exists.\n"
#             print_msg("ERROR", msg, debug_level)
#             valid = False
#         elif valid:
#             msg = f"It is valid to create Category `{category_name}` with:\n"
#             print_msg("ALWAYS", msg, debug_level)
#
#     if directive == "display":
#         print(Markdown(category_display))
#         return None
#
#     elif directive == "validate":
#         if valid:
#             print(Markdown(category_display))
#         else:
#             msg = f"Validation failed for {object_type} `{category_name}`\n"
#             print_msg(ERROR, msg, debug_level)
#             print(Markdown(category_display))
#         return valid
#
#     elif directive == "process":
#         if valid:
#             print(Markdown(category_display))
#         else:
#             if category_exists and object_action == "Create":
#                 msg = (f"Create failed because category `{category_name}` exists - changing `Create` to `Update` in "
#                        f"processed output \n")
#                 print_msg(ERROR, msg, debug_level)
#                 print(Markdown(category_display))
#                 return update_a_command(txt, object_action, object_type, known_q_name, known_guid)
#             else:
#                 return None
#
#         if object_action == "Update":
#             if not category_exists:
#                 print(f"\n{ERROR}category `{category_name}` does not exist! Updating result document with Create "
#                       f"object_action\n")
#                 return update_a_command(txt, object_action, object_type, known_q_name, known_guid)
#
#             # Update the basic category properties
#             egeria_client.update_category(known_guid, category_name, description, known_q_name, None)
#             msg = f"->Updated category `{category_name}`with GUID {known_guid}"
#             print_msg(ALWAYS, msg, debug_level)
#
#             # Update parent-child relationships
#
#             update_element_dictionary(known_q_name, {
#                 'guid': known_guid, 'display_name': category_name
#                 })
#
#             category_sync = update_category_parent(egeria_client, known_guid, parent_category_name)
#             print_msg(ALWAYS, f"Updated Category hierarchy for  `{category_name}` with outcome {category_sync}",
#                       debug_level)
#             return egeria_client.get_category_by_guid(known_guid, output_format='FORM')
#
#         elif object_action == "Create":
#             is_root = True
#
#             if category_exists:
#                 msg = (f"Cannot create`{category_name}` because it already exists; result document written for "
#                        f"category update\n")
#                 print_msg(WARNING, msg, debug_level)
#                 return update_a_command(txt, object_action, object_type, known_q_name, known_guid)
#             else:
#                 category_guid = egeria_client.create_category(known_glossary_guid, category_name, description, is_root)
#                 category_details = egeria_client.get_category_by_guid(category_guid)
#
#                 if category_details == NO_CATEGORIES_FOUND:
#                     msg = f"Just created category with GUID {category_guid} but category not found"
#                     print_msg(ERROR, msg, debug_level)
#                     return None
#
#                 qualified_name = category_details['glossaryCategoryProperties']["qualifiedName"]
#                 update_element_dictionary(qualified_name, {
#                     'guid': category_guid, 'display_name': category_name
#                     })
#                 print_msg(ALWAYS, f"Created Category `{category_name}` with GUID {category_guid}", debug_level)
#                 if parent_valid and parent_guid:
#                     egeria_client.set_parent_category(parent_guid, category_guid)
#                     print_msg(ALWAYS, f"Set parent category for `{category_name}` to `{parent_category_name}`",
#                               debug_level)
#                 else:
#                     print_msg(ERROR,
#                               f"Parent category `{parent_category_name}` not found or invalid for `{category_name}`",
#                               debug_level)
#                 return egeria_client.get_category_by_guid(category_guid, output_format='FORM')
#         return None
#     return None
#
#
def update_category_parent(egeria_client, category_guid: str, parent_category_name: str = None) -> bool:
    pass
#     """
#     Updates the parent relationship for a category.
#
#     If a parent category is specified, it will check if a parent is currently set.
#     If a parent category was set and is the same as the parent category specified, no change is needed.
#     If a parent category was set and is different from the parent_category_name, the parent category is updated.
#     If parent_category_name is None or empty and an existing parent category was set, the parent category is removed.
#
#     Parameters
#     ----------
#     egeria_client: EgeriaTech
#         The Egeria client to use for API calls
#     category_guid: str
#         The GUID of the category to update
#     parent_category_name: str, optional
#         The name of the parent category to set, or None to remove the parent
#
#     Returns
#     -------
#
#     True if successful, False otherwise.
#
#     """
#     outcome = True
#     # Handle parent category updates
#     if parent_category_name:
#         # Check if a parent is currently set
#         current_parent = egeria_client.get_category_parent(category_guid)
#
#         if isinstance(current_parent, str) and "No Parent Category found" in current_parent:
#             # No parent currently set, need to set it
#             _, parent_guid, _, parent_exists = get_element_by_name(egeria_client, 'Glossary Categories',
#                                                                    parent_category_name)
#
#             if parent_exists and parent_guid:
#                 egeria_client.set_parent_category(parent_guid, category_guid)
#                 print_msg(ALWAYS, f"Set parent category of category to `{parent_category_name}`", debug_level)
#             else:
#                 print_msg(ERROR, f"Parent category `{parent_category_name}` not found", debug_level)
#                 outcome = False
#         else:
#             # Parent is set, check if it's the same
#             current_parent_name = current_parent.get('glossaryCategoryProperties', {}).get('qualifiedName', '')
#
#             if current_parent_name != parent_category_name:
#                 # Different parent, need to update
#                 # First remove the current parent
#                 current_parent_guid = current_parent.get('elementHeader', {}).get('guid', '')
#                 if current_parent_guid:
#                     egeria_client.remove_parent_category(current_parent_guid, category_guid)
#
#                 # Then set the new parent
#                 _, parent_guid, _, parent_exists = get_element_by_name(egeria_client, 'Glossary Categories',
#                                                                        parent_category_name)
#
#                 if parent_exists and parent_guid:
#                     egeria_client.set_parent_category(parent_guid, category_guid)
#                     print_msg(ALWAYS,
#                               f"Updated parent category from `{current_parent_name}` to `{parent_category_name}`",
#                               debug_level)
#                 else:
#                     print_msg(ERROR, f"Parent category `{parent_category_name}` not found", debug_level)
#                     outcome = False
#     elif parent_category_name is None or parent_category_name == '':
#         # Check if a parent is currently set and remove it if needed
#         current_parent = egeria_client.get_category_parent(category_guid)
#
#         if not isinstance(current_parent, str) or "No Parent Category found" not in current_parent:
#             # Parent is set, need to remove it
#             current_parent_guid = current_parent.get('elementHeader', {}).get('guid', '')
#             current_parent_name = current_parent.get('glossaryCategoryProperties', {}).get('qualifiedName', '')
#
#             if current_parent_guid:
#                 egeria_client.remove_parent_category(current_parent_guid, category_guid)
#                 print_msg(ALWAYS, f"Removed parent category `{current_parent_name}`", debug_level)
#
#     return outcome


def process_term_upsert_command(egeria_client: EgeriaTech, txt: str, directive: str = "display") -> Optional[str]:
    pass
    """
    Processes a term create or update object_action by extracting key attributes such as
    term name, summary, description, abbreviation, examples, usage, version, and status from the given cell.

    :param txt: A string representing the input cell to be processed for
        extracting glossary-related attributes.
    :param directive: an optional string indicating the directive to be used - display, validate or execute
    :return: A string summarizing the outcome of the processing.
    """

    command, object_type, object_action = extract_command_plus(txt)
    print(Markdown(f"# {command}\n"))

    parsed_output = parse_upsert_command(egeria_client, object_type, object_action, txt, directive)
    if not parsed_output:
        logger.error(f"No output for `{object_action}`")
        return None

    valid = parsed_output['valid']
    exists = parsed_output['exists']

    qualified_name = parsed_output.get('qualified_name', None)
    guid = parsed_output.get('guid', None)

    print(Markdown(parsed_output['display']))

    logger.debug(json.dumps(parsed_output, indent=4))

    attributes = parsed_output['attributes']

    display_name = attributes['Display Name'].get('value', None)
    status = attributes.get('Status', {}).get('value', None)
    merge_update = attributes.get('Merge Update', {}).get('value', True)
    #

    if directive == "display":
        return None
    elif directive == "validate":
        if valid:
            print(Markdown(f"==> Validation of {command} completed successfully!\n"))
        else:
            msg = f"Validation failed for object_action `{command}`\n"
        return valid

    elif directive == "process":
        try:
            obj = "GlossaryTerm"
            #   Set the property body for a glossary collection
            #
            prop_body = set_element_prop_body(obj, qualified_name, attributes)
            prop_body["aliases"] = attributes.get('Aliases', {}).get('value', None)
            prop_body["summary"] = attributes.get('Summary', {}).get('value', None)
            prop_body["examples"] = attributes.get('Examples', {}).get('value', None)
            prop_body["abbreviation"] = attributes.get('Abbreviation', {}).get('value', None)
            prop_body["usage"] = attributes.get('Usage', {}).get('value', None)
            prop_body["user_defined_status"] = attributes.get('UserDefinedStatus', {}).get('value', None)
            to_be_collection_guids = [attributes.get("Glossary", {}).get("guid", {}),
                                      attributes.get("Folder", {}).get("guid_list", {})]
            glossary_guid = attributes.get("Glossary", {}).get('guid', None)
            anchor_scope_guid = attributes.get("Anchor Scope", {}).get('guid', None)


            get_method = egeria_client.get_term_by_guid
            collection_types = ["Glossary", "Folder"]

            output_set = make_format_set_name_from_type(object_type)

            if object_action == "Update":
                if not exists:
                    msg = (f" Element `{display_name}` does not exist! Updating result document with Create "
                           f"{object_action}\n")
                    logger.error(msg)
                    return update_a_command(txt, object_action, object_type, qualified_name, guid)
                elif not valid:
                    return None
                else:
                    print(Markdown(
                        f"==> Validation of {command} completed successfully! Proceeding to apply the changes.\n"))

                body = set_update_body(obj, attributes)
                body['properties'] = prop_body

                egeria_client.update_glossary_term(guid, body)
                if status:
                    egeria_client.update_term_status(guid, status)
                get_method = egeria_client.get_term_by_guid
                collection_types = ["Glossary", "Folder"]

                sync_collection_memberships(egeria_client, guid, get_method,  collection_types,
                                            to_be_collection_guids, merge_update)
                logger.success(f"Updated  {object_type} `{display_name}` with GUID {guid}\n\n___")
                update_element_dictionary(qualified_name, {
                    'guid': guid, 'display_name': display_name
                    })
                return egeria_client.get_term_by_guid(guid, element_type='GlossaryTerm',
                                                            output_format='MD', report_spec=output_set)


            elif object_action == "Create":
                if valid is False and exists:
                    msg = (f"  Digital Product `{display_name}` already exists and result document updated changing "
                           f"`Create` to `Update` in processed output\n\n___")
                    logger.error(msg)
                    return update_a_command(txt, object_action, object_type, qualified_name, guid)

                else:
                    body = set_create_body(obj,attributes)

                    # if this is a root or folder (maybe more in the future), then make sure that the classification is set.

                    body["properties"] = prop_body
                    # If the anchor scope is not already set and a glossary is specified
                    if anchor_scope_guid is None and glossary_guid:
                        body["anchorScopeGUID"] = glossary_guid

                    guid = egeria_client.create_glossary_term(body = body)
                    if guid:
                        sync_collection_memberships(egeria_client, guid, get_method,
                                                    collection_types, to_be_collection_guids, True)
                        update_element_dictionary(qualified_name, {
                            'guid': guid, 'display_name': display_name
                            })
                        msg = f"Created Element `{display_name}` with GUID {guid}\n\n___"
                        logger.success(msg)
                        return egeria_client.get_term_by_guid(guid, obj, output_format='MD', report_spec=output_set)
                    else:
                        msg = f"Failed to create element `{display_name}` with GUID {guid}\n\n___"
                        logger.error(msg)
                        return None

        except PyegeriaException as e:
            logger.error(f"Pyegeria error performing {command}: {e}")
            print_basic_exception(e)
            return None
        except Exception as e:
            logger.error(f"Error performing {command}: {e}")
    else:
        return None

def process_link_term_term_relationship_command(egeria_client: EgeriaTech, txt: str,
                                                  directive: str = "display") -> Optional[str]:
    """ Relate two terms through the specified relationship. ."""
    command, object_type, object_action = extract_command_plus(txt)
    print(Markdown(f"# {command}\n"))

    parsed_output = parse_view_command(egeria_client, object_type, object_action, txt, directive)

    print(Markdown(parsed_output['display']))

    logger.debug(json.dumps(parsed_output, indent=4))

    attributes = parsed_output['attributes']
    term1_guid = attributes.get('Term 1', {}).get('guid', None)
    term1_qname = attributes.get('Term 1', {}).get('qualified_name', None)
    term2_guid = attributes.get('Term 2', {}).get('guid', None)
    term2_qname = attributes.get('Term 2', {}).get('qualified_name', None)
    relationship = attributes.get('Relationship', {}).get('value', None)
    expression = attributes.get('Expression', {}).get('value', None)
    confidence = attributes.get('Confidence', {}).get('value', None)
    status = attributes.get('Status', {}).get('value', None)
    steward = attributes.get('Steward', {}).get('value', None)
    source = attributes.get('Source', {}).get('value', None)



    valid = parsed_output['valid']
    exists = term1_guid is not None and term2_guid is not None

    if not (valid and exists):
        valid = False

    if directive == "display":
        return None
    elif directive == "validate":
        if valid:
            print(Markdown(f"==> Validation of {command} completed successfully!\n"))
        else:
            msg = f"Validation failed for object_action `{command}`\n"
        return valid

    elif directive == "process":
        prop_body = set_rel_prop_body("GlossaryTerm", attributes)
        prop_body['expression'] = expression
        prop_body['confidence'] = confidence
        prop_body['status'] = status
        prop_body['steward'] = steward
        prop_body['source'] = source

        try:

            if not valid:  # First validate the term before we process it
                return None

            if object_action == "Link":
                if not exists:
                    msg = f"  Term `{term1_guid}` or {term2_guid} does not exist! "
                    logger.error(msg)
                    return None
                else:
                    print(Markdown(
                        f"==> Validation of {command} completed successfully! Proceeding to apply the changes.\n"))
            body = set_rel_request_body(relationship, prop_body)
            body['properties'] = prop_body
            body = body_slimmer(body)
            # todo removing body for now since don't know relationship properties class
            egeria_client.add_relationship_between_terms(term1_guid, term2_guid, relationship)
            logger.success(f"Created  `{relationship}` relationship between `{term1_qname}` and `{term2_qname}`\n\n___")
            update_md = (f"\n\n# Update Term-Term Relationship\n\n## Term 1 Name:\n\n{term1_qname}"
                         f"\n\n## Term 2 Name\n\n{term2_qname}\n\n## Term Relationship:\n\n{relationship}")
            return update_md


        except Exception as e:
            print(f"Error performing {command}: {e}")
            print_basic_exception(e)
            return None
            return None
    else:
        return None





"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

General utility functions in support of the Egeria Python Client package.

"""

import json
from datetime import datetime
from loguru import logger
from rich import print, print_json
from rich.console import Console
from pyegeria.config import settings as app_settings
from typing import Callable, TypeVar

T = TypeVar('T', bound=Callable)

console = Console(width=200)

def init_log():
    pass

def print_rest_request_body(body):
    """

    Args:
        body:
    """
    pretty_body = json.dumps(body, indent=4)
    print_json(pretty_body, indent=4, sort_keys=True)


def print_rest_response(response):
    """

    Args:
        response:
    """
    print("Returns:")
    pretty_body = json.dumps(response, indent=4)
    print_json(pretty_body, indent=4, sort_keys=True)


def print_guid_list(guids):
    """Print a list of guids"""
    if guids is None:
        print("No assets created")
    else:
        pretty_guids = json.dumps(guids, indent=4)
        print_json(pretty_guids, indent=4, sort_keys=True)


#
# OCF Common services
# Working with assets - this set of functions displays assets returned from the open metadata repositories.
#




def body_slimmer(body: dict) -> dict:
    """body_slimmer is a little function to remove unused keys from a dict
    and recursively slim embedded dicts

    Parameters
    ----------
    body : the dictionary that you want to slim

    Returns
    -------
    dict:
        a slimmed body with all embedded dictionaries also slimmed
    """
    if body is None:
        return {}

    slimmed = {}
    for key, value in body.items():
        if value and not isinstance(value, tuple):
            if isinstance(value, dict):
                # Recursively slim embedded dictionaries
                slimmed_value = body_slimmer(value)
                if slimmed_value:  # Only include non-empty dictionaries
                    slimmed[key] = slimmed_value
            else:
                slimmed[key] = value
    return slimmed

import re

def camel_to_title_case(input_string):
    # Add a space before uppercase letters and capitalize each word
    result = re.sub(r'([a-z])([A-Z])', r'\1 \2', input_string).title()
    return result


def to_camel_case(input_string):
    """Convert an input string to camelCase, singularizing if plural.
    
    This function takes an input string, converts it to singular form if it's plural,
    and then transforms it to camelCase format (first word lowercase, subsequent words
    capitalized with no spaces).
    
    Parameters
    ----------
    input_string : str
        The string to convert to camelCase
        
    Returns
    -------
    str:
        The input string converted to camelCase, after singularization if needed
        
    Examples
    --------
    >>> to_camel_case("data categories")
    'dataCategory'
    >>> to_camel_case("business terms")
    'businessTerm'
    >>> to_camel_case("glossary categories")
    'glossaryCategory'
    """
    if not input_string:
        return ""
    
    # Convert to lowercase for consistent processing
    lowercase_input = input_string.lower()
    
    # First, convert to singular if plural
    singular = lowercase_input
    
    # Handle common plural endings
    if singular.endswith('ies'):
        singular = singular[:-3] + 'y'
    elif singular.endswith('es'):
        # Special cases like 'classes' -> 'class'
        if singular.endswith('sses') or singular.endswith('ches') or singular.endswith('shes') or singular.endswith('xes'):
            singular = singular[:-2]
        else:
            singular = singular[:-1]
    elif singular.endswith('s') and not singular.endswith('ss'):
        singular = singular[:-1]
    
    # Split the string into words and convert to camelCase
    words = singular.split()
    if not words:
        return ""
    
    # First word is lowercase, rest are capitalized
    result = words[0]
    for word in words[1:]:
        result += word.capitalize()
    
    return result

def to_pascal_case(input_string)->str:
    """
        Convert input string to PascalCase, singularizing if plural.
    Args:
        input_string ():

    Returns:
        transformed string
    """
    result = to_camel_case(input_string)
    output_string = result[0].upper() + result[1:]
    return output_string

def flatten_dict_to_string(d: dict) -> str:
    """Flatten a dictionary into a string and replace quotes with backticks."""
    try:
        flat_string = ", ".join(
            # Change replace(\"'\", '`') to replace("'", '`')
            f"{key}=`{str(value).replace('\"', '`').replace("'", '`')}`"
            for key, value in d.items()
        )
        return flat_string
    except Exception as e:
        # Corrected syntax for exception chaining
        raise Exception("Error flattening dictionary") from e
# The decorator logic, which applies @logger.catch dynamically


import json
import re


# def parse_to_dict(input_str: str):
#     """
#     Check if a string is valid JSON or a name:value list without braces and convert to a dictionary.
#
#     Args:
#         input_str: The input string to parse.
#
#     Returns:
#         dict: A dictionary converted from the input string.
#         None: If the input is neither valid JSON nor a valid name:value list.
#     """
#
#     if input_str is None:
#         return None
#
#     # Check if the input string is valid JSON
#     try:
#         result = json.loads(input_str)
#         if isinstance(result, dict):  # Ensure it's a dictionary
#             return result
#     except json.JSONDecodeError:
#         pass
#
#     # Check if input string looks like a name:value list
#     # Supports both comma and newline as separators
#     pattern = r'^(\s*("[^"]+"|\'[^\']+\'|[a-zA-Z0-9_-]+)\s*:\s*("[^"]+"|\'[^\']+\'|[a-zA-Z0-9 _-]*)\s*)' \
#               r'(\s*[,|\n]\s*("[^"]+"|\'[^\']+\'|[a-zA-Z0-9_-]+)\s*:\s*("[^"]+"|\'[^\']+\'|[a-zA-Z0-9 _-]*)\s*)*$'
#     if re.match(pattern, input_str.strip()):
#         try:
#             # Split by ',' or '\n' and process key-value pairs
#             pairs = [pair.split(":", 1) for pair in re.split(r'[,|\n]+', input_str.strip())]
#             return {key.strip().strip('\'"'): value.strip().strip('\'"') for key, value in pairs}
#         except Exception:
#             return None
#
#     # If neither pattern matches, return None
#     return None


def parse_to_dict(input_str: str) -> dict | None:
    """
    Parse input strings into a dictionary, handling both JSON and key-value pairs.
    Recovers from malformed JSON (e.g., where commas are missing between key-value pairs)
    and supports multiline values.

    Args:
        input_str (str): The input string to parse.

    Returns:
        dict: A parsed dictionary if validation is successful, or None if the string cannot be parsed.
    """
    if not input_str:
        return None

    # Attempt to parse valid JSON
    try:
        result = json.loads(input_str)
        if isinstance(result, dict):
            return result
    except json.JSONDecodeError:
        pass

    # Fix malformed JSON or attempt alternate parsing for "key: value" patterns
    try:
        # Step 1: Inject missing commas where they are omitted between key-value pairs
        fixed_input = re.sub(
            r'("\s*:[^,}\n]+)\s*("(?![:,}\n]))',  # Find missing commas (key-value-value sequences)
            r'\1,\2',  # Add a comma between the values
            input_str
        )

        # Attempt to parse the fixed string as JSON
        try:
            result = json.loads(fixed_input)
            if isinstance(result, dict):
                return result
        except json.JSONDecodeError:
            pass

        # Step 2: Handle key-value format fallback (supports multiline strings)
        # Matches `key: value` pairs, including multiline quoted values
        key_value_pattern = re.compile(r'''
            (?:"([^"]+)"|'([^']+)'|([a-zA-Z0-9_-]+))  # Key: quoted "key", 'key', or unquoted key
            \s*:\s*                                   # Key-value separator
            (?:"((?:\\.|[^"\\])*?)"|'((?:\\.|[^'\\])*?)'|([^\n,]+))  # Value: quoted or unquoted
        ''', re.VERBOSE | re.DOTALL)

        matches = key_value_pattern.findall(input_str)

        # Build dictionary from matches
        result_dict = {}
        for match in matches:
            key = next((group for group in match[:3] if group), "").strip()
            value = next((group for group in match[3:] if group), "").strip()
            result_dict[key] = value

        if result_dict:
            return result_dict
    except Exception as e:
        # Log or handle parsing exception if needed
        pass

    # If all parsing attempts fail, return None
    return None


def dynamic_catch(func: T) -> T:
    if app_settings.get("enable_logger_catchh", False):
        return logger.catch(func)  # Apply the logger.catch decorator
    else:
        return func  # Return the function unwrapped

def make_format_set_name_from_type(obj_type: str)-> str:
    formatted_name = obj_type.replace(" ", "-")
    return f"{formatted_name}-DrE"


if __name__ == "__main__":
    print("Main-Utils")



"""This displays deployed databases"""

import argparse
import os
import sys
import time

from rich import box
from rich.console import Console
from rich.markdown import Markdown
from rich.table import Table

from pyegeria import (
    EgeriaTech,
    PyegeriaException,
    print_basic_exception,
    settings,
    config_logging
)



EGERIA_USER = os.environ.get("EGERIA_USER", "erinoverview")
EGERIA_USER_PASSWORD = os.environ.get("EGERIA_USER_PASSWORD", "secret")

app_config = settings.Environment
console = Console(width = app_config.console_width)
config_logging()

def check_if_template(header: dict) -> bool:
    """Check if the the template classification is set"""
    classifications = header.get("classifications", None)
    if classifications is None:
        return False
    for c in classifications:
        if c["type"]["typeName"] == "Template":
            return True
    return False


def list_deployed_databases(
    view_server: str = app_config.egeria_view_server,
    view_url: str = app_config.egeria_view_server_url,
    user: str = EGERIA_USER,
    user_pass: str = EGERIA_USER_PASSWORD,
    jupyter: bool = app_config.egeria_jupyter,
    width: int = app_config.console_width,
):
    """
    Parameters
    ----------
    view_server : str
        The view server name to which the Egeria client will connect. By default, this is set to EGERIA_VIEW_SERVER.
    view_url : str
        The URL of the Egeria view server. By default, this is set to EGERIA_VIEW_SERVER_URL.
    user : str
        The username used for authentication with the Egeria server. By default, it is set to EGERIA_USER.
    user_pass : str
        The password used for authentication with the Egeria server. By default, it is set to EGERIA_USER_PASSWORD.
    jupyter : bool
        A flag indicating whether the output is displayed within a Jupyter notebook. Default is set to EGERIA_JUPYTER.
    width : int
        The width of the output console. By default, it is set to EGERIA_WIDTH.
    """
    c_client = EgeriaTech(view_server, view_url, user_id=user, user_pwd=user_pass)
    token = c_client.create_egeria_bearer_token()

    def generate_table() -> Table:
        """Make a new table."""
        table = Table(
            title=f"Database List @ {time.asctime()}",
            caption=f"Databases found: {view_url} - {view_server} @ {time.asctime()}",
            style="bold bright_white on black",
            row_styles=["bold bright_white on black"],
            header_style="white on dark_blue",
            title_style="bold bright_white on black",
            caption_style="white on black",
            show_lines=True,
            box=box.ROUNDED,
            # title=f"Elements for Open Metadata Type: '{om_type}' ",
            expand=True,
            # width=500
        )

        table.add_column("Qualified Name")
        table.add_column("Type")
        table.add_column("Created", width=23)
        table.add_column("GUID", width=38, no_wrap=True)
        table.add_column("Properties")
        table.add_column("Schemas")

        om_type = "Database"
        dbs = c_client.get_elements(om_type)
        if type(dbs) is list:
            for element in dbs:
                header = element["elementHeader"]

                if check_if_template(header):
                    continue

                el_q_name = element["properties"].get("qualifiedName", "---")
                el_type = header["type"]["typeName"]
                el_home = header["origin"]["homeMetadataCollectionName"]
                el_create_time = header["versions"]["createTime"][:-10]
                el_created_by = header["versions"]["createdBy"]
                el_created_md = (
                    f"* **Created By**: {el_created_by}\n"
                    f"* **Created Time**: {el_create_time}\n"
                    f"* **Home Store**: {el_home}"
                )
                el_created_out = Markdown(el_created_md)

                el_guid = header["guid"]

                el_props_md = ""
                for prop in element["properties"].keys():
                    el_props_md += f"* **{prop}**: {element['properties'][prop]}\n"
                el_props_out = Markdown(el_props_md)

                rel_elements = c_client.get_related_elements(el_guid)
                schema_md = ""
                count = 0
                rel_cnt = len(rel_elements)
                if type(rel_elements) is list:
                    for rel_element in rel_elements:
                        count += 1
                        rel_type = rel_element["relatedElement"]["elementHeader"][
                            "type"
                        ]["typeName"]
                        rel_guid = rel_element["relatedElement"]["elementHeader"][
                            "guid"
                        ]
                        rel_props = rel_element["relatedElement"]["properties"]
                        props_md = ""
                        for key in rel_props.keys():
                            props_md += f"* **{key}**: {rel_props[key]}\n"
                        rel_el_md = f"* **{rel_type}**: {rel_guid}\n{props_md}\n---\n"
                        if count == rel_cnt:
                            rel_el_md = rel_el_md[:-4]
                    rel_el_out = Markdown(rel_el_md)

                table.add_row(
                    el_q_name,
                    el_type,
                    el_created_out,
                    el_guid,
                    el_props_out,
                    rel_el_out,
                )

            return table
        else:
            print("No instances found")
            sys.exit(1)

    try:
        console = Console(width=width, force_terminal=not jupyter)

        with console.pager(styles=True):
            console.print(generate_table())

    except (
        PyegeriaException
    ) as e:
        print_basic_exception(e)
        print("\n\nPerhaps the type name isn't known")
    finally:
        c_client.close_session()


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--server", help="Name of the server to display status for")
    parser.add_argument("--url", help="URL Platform to connect to")
    parser.add_argument("--userid", help="User Id")
    parser.add_argument("--password", help="Password")

    args = parser.parse_args()

    server = args.server if args.server is not None else app_config.egeria_view_server
    url = args.url if args.url is not None else app_config.egeria_view_server_url
    userid = args.userid if args.userid is not None else EGERIA_USER
    password = args.password if args.password is not None else EGERIA_USER_PASSWORD

    try:
        list_deployed_databases(server, url, userid, password)
    except KeyboardInterrupt:
        pass


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.



Egeria Coco Pharmaceutical demonstration labs.

This script creates and configures the Data Lake


"""


import argparse
import json

import httpx
from globals import (
    adminUserId,
    cocoCohort,
    cocoMDS1Name,
    cocoMDS2Name,
    cocoMDS4Name,
    corePlatformURL,
    dataLakePlatformURL,
    devCohort,
    fileSystemRoot,
    iotCohort,
    max_paging_size,
)

from pyegeria import (
    CoreServerConfig,
    FullServerConfig,
    Platform,
    print_exception_response,
)


def config_coco_datalake(url: str, userid: str):
    print("Configuring and starting the Data Lake")
    disable_ssl_warnings = True
    platform_url = url
    admin_user = userid

    #   Change the kafka configuration to reflect the distinguished kafka setting for the labs
    # event_bus_config = {
    #     "producer": {
    #         "bootstrap.servers": "{{kafkaEndpoint}}"
    #     },
    #     "consumer": {
    #         "bootstrap.servers": "{{kafkaEndpoint}}"
    #     }
    # }
    event_bus_config = {
        "producer": {"bootstrap.servers": "host.docker.internal:9192"},
        "consumer": {"bootstrap.servers": "host.docker.internal:9192"},
    }

    security_connection_body = {
        "class": "Connection",
        "connectorType": {
            "class": "ConnectorType",
            "connectorProviderClassName": "org.odpi.openmetadata.metadatasecurity.samples.CocoPharmaServerSecurityProvider",
        },
    }

    #
    # Configure MDS1
    #

    mdr_server = cocoMDS1Name
    mdr_server_user_id = "cocoMDS1npa"
    mdr_server_password = "cocoMDS1passw0rd"
    metadataCollectionId = f"{mdr_server}-e915f2fa-aa3g-4396-8bde-bcd65e642b1d"
    metadataCollectionName = "Data Lake Operations"

    print("Configuring " + mdr_server + "...")
    try:
        o_client = CoreServerConfig(mdr_server, platform_url, admin_user)

        o_client.set_basic_server_properties(
            metadataCollectionName,
            "Coco Pharmaceuticals",
            platform_url,
            mdr_server_user_id,
            mdr_server_password,
            max_paging_size,
        )

        o_client.set_event_bus(event_bus_config)
        o_client.set_server_security_connection(security_connection_body)
        o_client.add_default_log_destinations()

        # o_client.set_in_mem_local_repository()
        o_client.set_xtdb_local_kv_repository()

        o_client.set_local_metadata_collection_id(metadataCollectionId)
        o_client.set_local_metadata_collection_name(metadataCollectionName)

        o_client.add_cohort_registration(cocoCohort)
        access_service_options = {
            "SupportedZones": [
                "quarantine",
                "clinical-trials",
                "research",
                "data-lake",
                "trash-can",
            ]
        }

        # o_client.configure_access_service("asset-catalog", {})
        o_client.configure_access_service("asset-consumer", {})

        access_service_options["DefaultZones"] = ["quarantine"]
        access_service_options["PublishZones"] = ["data-lake"]

        # print(f"Access Service Options: {access_service_options}")

        o_client.configure_access_service("asset-manager", access_service_options)
        o_client.configure_access_service("asset-owner", access_service_options)
        o_client.configure_access_service(
            "community-profile", {"KarmaPointPlateau": "500"}
        )
        # o_client.configure_access_service("glossary-view", {})
        # o_client.configure_access_service("data-engine", access_service_options)
        o_client.configure_access_service("data-manager", access_service_options)
        o_client.configure_access_service(
            "digital-architecture", access_service_options
        )
        o_client.configure_access_service("governance-engine", access_service_options)
        o_client.configure_access_service("governance-server", access_service_options)
        o_client.configure_access_service("asset-lineage", access_service_options)

        print(f"Activating {mdr_server}")
        p_client = Platform(mdr_server, platform_url, admin_user)
        p_client.activate_server_stored_config()
        print(mdr_server + " activated")

    except Exception as e:
        print_exception_response(e)

    #
    # Configure MDS4
    #
    mdr_server = cocoMDS4Name
    mdr_server_user_id = "cocoMDS4npa"
    mdr_server_password = "cocoMDS4passw0rd"
    metadataCollectionId = f"{mdr_server}-e915f2fa-aa3g-4396-8bde-bcd65e642b1d"
    metadataCollectionName = "Data Lake Users"
    print("Configuring " + mdr_server + "...")

    try:
        o_client = CoreServerConfig(mdr_server, platform_url, admin_user)

        o_client.set_basic_server_properties(
            "Data Lake Users",
            "Coco Pharmaceuticals",
            platform_url,
            mdr_server_user_id,
            mdr_server_password,
            max_paging_size,
        )

        o_client.set_event_bus(event_bus_config)
        o_client.set_server_security_connection(security_connection_body)
        o_client.add_default_log_destinations()

        # Note: no metadata repository or collection configuration in this server.

        o_client.add_cohort_registration(cocoCohort)

        accessServiceOptions = {"SupportedZones": ["data-lake"]}

        # o_client.configure_access_service("asset-catalog", accessServiceOptions)
        o_client.configure_access_service("asset-consumer", accessServiceOptions)
        o_client.configure_access_service("asset-owner", {})
        o_client.configure_access_service(
            "community-profile", {"KarmaPointPlateau": "500"}
        )
        # o_client.configure_access_service("glossary-view", {})
        o_client.configure_access_service("data-science", accessServiceOptions)

        print(f"Activating {mdr_server}")
        p_client = Platform(mdr_server, platform_url, admin_user)
        p_client.activate_server_stored_config()

        print(f"{mdr_server} activated")

    except Exception as e:
        print_exception_response(e)

    #
    # Configure exchangeDL01
    #

    daemon_server_name = "exchangeDL01"
    daemon_server_platform = platform_url
    daemon_server_user_id = "exchangeDL01npa"
    daemon_server_password = "exchangeDL01passw0rd"

    mdr_server = "cocoMDS1"
    folder_connector_name = "DropFootClinicalTrialResultsFolderMonitor"
    folder_connector_user_id = "monitorDL01npa"
    folder_connector_source_name = "DropFootClinicalTrialResults"
    folder_connector_folder = (
        fileSystemRoot
        + "/data-lake/research/clinical-trials/drop-foot/weekly-measurements"
    )
    folder_connector_connection = {
        "class": "Connection",
        "connectorType": {
            "class": "ConnectorType",
            "connectorProviderClassName": "org.odpi.openmetadata.adapters.connectors.integration.basicfiles.DataFolderMonitorIntegrationProvider",
        },
        "endpoint": {"class": "Endpoint", "address": folder_connector_folder},
    }

    integration_group_name = "Onboarding"

    print("Configuring " + daemon_server_name)

    try:
        f_client = FullServerConfig(
            daemon_server_name, daemon_server_platform, admin_user
        )

        f_client.set_basic_server_properties(
            "Supports exchange of metadata with third party technologies",
            "Coco Pharmaceuticals",
            daemon_server_platform,
            daemon_server_user_id,
            daemon_server_password,
            max_paging_size,
        )

        f_client.set_server_security_connection(security_connection_body)
        f_client.add_default_log_destinations()

        connector_configs = [
            {
                "class": "IntegrationConnectorConfig",
                "connectorName": folder_connector_name,
                "connectorUserId": folder_connector_user_id,
                "connection": folder_connector_connection,
                "metadataSourceQualifiedName": folder_connector_source_name,
                "refreshTimeInterval": 10,
                "usesBlockingCalls": "false",
            }
        ]

        f_client.config_integration_service(
            mdr_server, platform_url, "files-integrator", {}, connector_configs
        )

        f_client.config_integration_group(
            mdr_server, daemon_server_platform, integration_group_name
        )
        print(f"Activating {daemon_server_name}")
        p_client = Platform(daemon_server_name, daemon_server_platform, adminUserId)
        p_client.activate_server_stored_config()

        print(f"{daemon_server_name} activated")

    except Exception as e:
        print_exception_response(e)

    #
    # Configure governDL01
    #
    engine_server = "governDL01"
    engine_server_platform = platform_url

    engine_server_user_id = "governDL01npa"
    engine_server_password = "governDL01passw0rd"
    mdr_server = "cocoMDS1"
    mdr_engine_server_platform = dataLakePlatformURL

    print("Configuring " + engine_server)

    try:
        o_client = CoreServerConfig(engine_server, engine_server_platform, admin_user)

        o_client.set_basic_server_properties(
            "An Engine Host to run governance actions for Coco Pharmaceuticals",
            "Coco Pharmaceuticals",
            engine_server_platform,
            engine_server_user_id,
            engine_server_password,
            max_paging_size,
        )

        o_client.set_server_security_connection(security_connection_body)

        o_client.set_engine_definitions_client_config(
            mdr_server, mdr_engine_server_platform
        )

        engine_list_body = [
            {
                "class": "EngineConfig",
                "engineQualifiedName": "AssetDiscovery",
                "engineUserId": "findItDL01npa",
            },
            {
                "class": "EngineConfig",
                "engineQualifiedName": "AssetQuality",
                "engineUserId": "findItDL01npa",
            },
        ]

        o_client.set_engine_list(engine_list_body)

        # config = o_client.get_stored_configuration()
        # print(f"The server stored configuration is {json.dumps(config, indent=4)}")
        print(f"Activating {engine_server}")
        p_client = Platform(engine_server, engine_server_platform, admin_user)
        p_client.activate_server_stored_config()
        print(f"{engine_server} activated")
    except Exception as e:
        print_exception_response(e)

    #
    # Configure cocoView1
    #

    view_server = "cocoView1"
    view_server_user_id = "cocoView1npa"
    view_server_password = "cocoView1passw0rd"
    view_server_type = "View Server"
    remote_platform_url = corePlatformURL
    remote_server_name = cocoMDS1Name

    print("Configuring " + view_server)
    try:
        f_client = FullServerConfig(view_server, platform_url, admin_user)

        f_client.set_server_user_id(view_server_user_id)
        f_client.set_server_user_password(view_server_password)
        f_client.set_organization_name("Coco Pharmaceuticals")

        f_client.set_server_description("Coco View Server")
        f_client.set_server_url_root(platform_url)
        f_client.set_event_bus(event_bus_config)
        f_client.set_server_security_connection(security_connection_body)

        f_client.add_default_log_destinations()

        f_client.config_all_view_services(remote_server_name, platform_url)
        print(f"Activating {view_server}")
        p_client = Platform(view_server, platform_url, admin_user)
        p_client.activate_server_stored_config()

        print(f"{view_server} activated")

    except Exception as e:
        print_exception_response(e)

    #
    # Configure cocoOLS1
    #

    lineageServerName = "cocoOLS1"
    lineageServerPlatform = dataLakePlatformURL

    mdrServerName = "cocoMDS1"
    mdrServerUserId = "cocoMDS1npa"
    mdrServerPassword = "secret"
    mdrServerPlatform = dataLakePlatformURL

    print("Configuring " + lineageServerName)

    requestBody = {
        "class": "OpenLineageConfig",
        "openLineageDescription": "Open Lineage Service is used for the storage and querying of lineage",
        "lineageGraphConnection": {
            "class": "Connection",
            "displayName": "Lineage Graph Connection",
            "description": "Used for storing lineage in the Open Metadata format",
            "connectorType": {
                "class": "ConnectorType",
                "connectorProviderClassName": "org.odpi.openmetadata.openconnectors.governancedaemonconnectors.lineagewarehouseconnectors.janusconnector.graph.LineageGraphConnectorProvider",
            },
            "configurationProperties": {
                "gremlin.graph": "org.janusgraph.core.JanusGraphFactory",
                "storage.backend": "berkeleyje",
                "storage.directory": "data/servers/"
                + lineageServerName
                + "/lineage-repository/berkeley",
                "index.search.backend": "lucene",
                "index.search.directory": "data/servers/"
                + lineageServerName
                + "/lineage-repository/searchindex",
            },
        },
        "accessServiceConfig": {
            "serverName": mdrServerName,
            "serverPlatformUrlRoot": mdrServerPlatform,
            "user": mdrServerUserId,
            "password": mdrServerPassword,
        },
        "backgroundJobs": [
            {"jobName": "LineageGraphJob", "jobInterval": 120, "jobEnabled": "false"},
            {
                "jobName": "AssetLineageUpdateJob",
                "jobInterval": 120,
                "jobEnabled": "false",
                "jobDefaultValue": "2021-12-03T10:15:30",
            },
        ],
    }

    try:
        f_client = FullServerConfig(
            lineageServerName, lineageServerPlatform, admin_user
        )

        f_client.set_server_description("Coco View Server")
        f_client.set_server_url_root(lineageServerPlatform)
        f_client.set_event_bus(event_bus_config)
        f_client.add_default_log_destinations()

        f_client.set_lineage_warehouse_services(requestBody, lineageServerName)
        print(f"Activating {lineageServerName}")
        p_client = Platform(lineageServerName, lineageServerPlatform, admin_user)
        p_client.activate_server_stored_config()

        print(f"{lineageServerName} activated")

    except Exception as e:
        print_exception_response(e)


def main():
    parser = argparse.ArgumentParser()

    parser.add_argument("--url", help="URL Platform to connect to")
    parser.add_argument("--userid", help="User Id")
    args = parser.parse_args()

    url = args.url if args.url is not None else dataLakePlatformURL
    userid = args.userid if args.userid is not None else adminUserId

    config_coco_datalake(url, userid)


if __name__ == "__main__":
    main()



"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

Client to configure the Egeria platform and servers

"""

from pyegeria.full_omag_server_config import FullServerConfig
from pyegeria.server_operations import  ServerOps


class EgeriaConfig:
    """
    Client for configuring the Egeria Platform and Servers using composition.

    Attributes:
        server_name: str
            Name of the server to use.
        platform_url : str
            URL of the server platform to connect to
        user_id : str
            The identity of the user calling the method - this sets a default optionally used by the methods
            when the user doesn't pass the user_id on a method call.
        user_pwd: str
            The password associated with the user_id. Defaults to None

    Methods:
        Methods are provided by composed sub-clients via delegation.
    """

    def __init__(
        self, server_name: str, platform_url: str, user_id: str, user_pwd: str = None
    ):
        self.server_name = server_name
        self.platform_url = platform_url
        self.user_id = user_id
        self.user_pwd = user_pwd

        self._config = FullServerConfig(server_name, platform_url, user_id, user_pwd)
        self._ops = ServerOps(server_name, platform_url, user_id, user_pwd)
        self._subclients = [self._config, self._ops]

    def __getattr__(self, name):
        for sub in self._subclients:
            if hasattr(sub, name):
                return getattr(sub, name)
        raise AttributeError(f"{self.__class__.__name__!s} object has no attribute {name!r}")

    def create_egeria_bearer_token(self, user_id: str = None, user_pwd: str = None):
        token_val = None
        for sub in self._subclients:
            token_val = sub.create_egeria_bearer_token(user_id, user_pwd)
        return token_val

    def set_bearer_token(self, token: str) -> None:
        for sub in self._subclients:
            sub.set_bearer_token(token)

    def get_token(self) -> str:
        for sub in self._subclients:
            if hasattr(sub, "get_token"):
                return sub.get_token()
        return None

    def close_session(self) -> None:
        for sub in self._subclients:
            if hasattr(sub, "close_session"):
                try:
                    sub.close_session()
                except Exception:
                    pass


if __name__ == "__main__":
    print("Main-Config Client")



"""
This file contains project-related object_action functions for processing Egeria Markdown
"""

import json
import os
import re
from typing import List, Optional

from inflect import engine
from loguru import logger
from pydantic import ValidationError
from rich import print
from rich.console import Console
from rich.markdown import Markdown

from md_processing.md_processing_utils.common_md_proc_utils import (parse_upsert_command, parse_view_command,
                                                                    sync_collection_memberships)
from md_processing.md_processing_utils.common_md_utils import update_element_dictionary, set_update_body, \
    set_element_status_request_body, set_element_prop_body, set_delete_request_body, set_rel_request_body, \
    set_peer_gov_def_request_body, \
    set_rel_request_body, set_create_body, set_object_classifications, set_product_body, set_rel_request_body_for_type


from pyegeria import DEBUG_LEVEL, body_slimmer, to_pascal_case, PyegeriaException, print_basic_exception, \
    print_exception_table, print_validation_error
from pyegeria.egeria_tech_client import EgeriaTech



from md_processing.md_processing_utils.common_md_utils import (debug_level, print_msg, set_debug_level,
                                                               get_element_dictionary, update_element_dictionary,
                                                               )
from md_processing.md_processing_utils.extraction_utils import (extract_command_plus, extract_command,
                                                                process_simple_attribute, process_element_identifiers,
                                                                update_a_command, extract_attribute,
                                                                get_element_by_name, process_name_list)

from pyegeria._globals import (NO_GLOSSARIES_FOUND, NO_ELEMENTS_FOUND, NO_CATEGORIES_FOUND)
from pyegeria.egeria_tech_client import EgeriaTech
from pyegeria.utils import make_format_set_name_from_type, body_slimmer


# EGERIA_WIDTH = int(os.environ.get("EGERIA_WIDTH", "170"))
# console = Console(width=EGERIA_WIDTH)



def process_project_upsert_command(egeria_client: EgeriaTech, txt: str, directive: str = "display") -> Optional[str]:
    """
    Processes a project create or update object_action by extracting key attributes such as
    glossary name, language, description, and usage from the given text.

    :param txt: A string representing the input cell to be processed for
        extracting glossary-related attributes.
    :param directive: an optional string indicating the directive to be used - display, validate or execute
    :return: A string summarizing the outcome of the processing.
    """

    command, object_type, object_action = extract_command_plus(txt)
    print(Markdown(f"# {command}\n"))

    parsed_output = parse_upsert_command(egeria_client, object_type, object_action, txt, directive)
    if not parsed_output:
        logger.error(f"No output for `{object_action}`")
        return None

    valid = parsed_output['valid']
    exists = parsed_output['exists']

    qualified_name = parsed_output.get('qualified_name', None)
    guid = parsed_output.get('guid', None)

    print(Markdown(parsed_output['display']))

    logger.debug(json.dumps(parsed_output, indent=4))

    attributes = parsed_output['attributes']

    display_name = attributes['Display Name'].get('value', None)
    status = attributes.get('Status', {}).get('value', None)
    output_set = make_format_set_name_from_type(object_type)
    #

    if directive == "display":
        return None
    elif directive == "validate":
        if valid:
            print(Markdown(f"==> Validation of {command} completed successfully!\n"))
        else:
            msg = f"Validation failed for object_action `{command}`\n"
        return valid

    elif directive == "process":
        try:
            obj = "Project"
            project_types = ["Campaign", "Task", "Personal Project", "Study Project"]


            #   Set the property body for a glossary collection
            #
            prop_body = set_element_prop_body(obj, qualified_name, attributes)
            prop_body["identifier"] = attributes.get('Identifier', {}).get('value', None)
            prop_body["mission"] = attributes.get('Mission', {}).get('value', None)
            prop_body["purposes"] = attributes.get('Purposes', {}).get('value', None)
            prop_body["startDate"] = attributes.get('Start Date', {}).get('value', None)
            prop_body["endDate"] = attributes.get('End Date', {}).get('value', None)
            prop_body["priority"] = attributes.get('Priority', {}).get('value', None)
            prop_body["projectPhase"] = attributes.get('Project Phase', {}).get('value', None)
            prop_body["projectStatus"] = attributes.get('Project Status', {}).get('value', None)
            prop_body["projectHealth"] = attributes.get('Project Health', {}).get('value', None)

            if object_action == "Update":
                if not exists:
                    msg = (f" Element `{display_name}` does not exist! Updating result document with Create "
                           f"{object_action}\n")
                    logger.error(msg)
                    return update_a_command(txt, object_action, object_type, qualified_name, guid)
                elif not valid:
                    msg = ("  The input data is invalid and cannot be processed. \nPlease review")
                    logger.error(msg)
                    print(Markdown(f"==> Validation of {command} failed!!\n"))
                    print(Markdown(msg))
                    return None
                else:
                    print(Markdown(
                        f"==> Validation of {command} completed successfully! Proceeding to apply the changes.\n"))


                body = set_update_body(obj, attributes)
                body['properties'] = prop_body

                egeria_client.update_project(guid, body)
                # if status:
                #     egeria_client.update_project_status(guid, status)

                logger.success(f"Updated  {object_type} `{display_name}` with GUID {guid}\n\n___")
                update_element_dictionary(qualified_name, {
                    'guid': guid, 'display_name': display_name
                    })
                return egeria_client.get_project_by_guid(guid, element_type='Project',
                                                            output_format='MD', report_spec = output_set)


            elif object_action == "Create":
                if valid is False and exists:
                    msg = (f"Project `{display_name}` already exists and result document updated changing "
                           f"`Create` to `Update` in processed output\n\n___")
                    logger.error(msg)
                    return update_a_command(txt, object_action, object_type, qualified_name, guid)
                elif not valid:
                    msg = ("The input data is invalid and cannot be processed. \nPlease review")
                    logger.error(msg)
                    print(Markdown(f"==> Validation of {command} failed!!\n"))
                    print(Markdown(msg))
                    return None

                else:
                    body = set_create_body(object_type,attributes)

                    # if this is a root or folder (maybe more in the future), then make sure that the classification is set.
                    body["initialClassifications"] = set_object_classifications(object_type, attributes, project_types)

                    body["properties"] = prop_body
                    slim_body = body_slimmer(body)
                    guid = egeria_client.create_project(body = slim_body)
                    if guid:
                        update_element_dictionary(qualified_name, {
                            'guid': guid, 'display_name': display_name
                            })
                        msg = f"Created Element `{display_name}` with GUID {guid}\n\n___"
                        logger.success(msg)
                        return egeria_client.get_project_by_guid(guid, output_format='MD', report_spec = output_set)
                    else:
                        msg = f"Failed to create element `{display_name}` with GUID {guid}\n\n___"
                        logger.error(msg)
                        return None

        except PyegeriaException as e:
            logger.error(f"Pyegeria error performing {command}: {e}")
            print_basic_exception(e)
            return None
        except Exception as e:
            logger.error(f"Error performing {command}: {e}")
    else:
        return None



def process_link_project_hierarchy_command(egeria_client: EgeriaTech, txt: str, directive: str = "display") -> Optional[str]:

#     """ Set one project to manage another."""
#
    command, object_type, object_action = extract_command_plus(txt)
    print(Markdown(f"# {command}\n"))

    parsed_output = parse_view_command(egeria_client, object_type, object_action,
                                       txt, directive)

    if not parsed_output:
        logger.error(f"No output for `{object_action}`")
        print(Markdown("## Parsing failed"))
        return None

    print(Markdown(parsed_output['display']))

    logger.debug(json.dumps(parsed_output, indent=4))

    attributes = parsed_output['attributes']

    parent_project_guid = attributes.get('Parent Project', {}).get('guid', None)
    child_project_guid = attributes.get('Child Project', {}).get('guid', None)
    label = attributes.get('Link Label', {}).get('value', "")

    valid = parsed_output['valid']
    exists = parent_project_guid is not None and child_project_guid  is not None

    if directive == "display":

        return None
    elif directive == "validate":
        if valid:
            print(Markdown(f"==> Validation of {command} completed successfully!\n"))
        else:
            msg = f"Validation failed for object_action `{command}`\n"
            logger.error(msg)
            print(Markdown(f"==> Validation of {command} failed!!\n"))
        return valid

    elif directive == "process":


        try:
            if object_action == "Detach":
                if not exists:
                    msg = (f" Link `{label}` does not exist! Updating result document with Link "
                           f"object_action\n")
                    logger.error(msg)
                    out = parsed_output['display'].replace('Link', 'Detach', 1)
                    return out
                elif not valid:
                    return None
                else:
                    print(Markdown(
                        f"==> Validation of {command} completed successfully! Proceeding to apply the changes.\n"))
                    body = set_delete_request_body(object_type, attributes)

                    egeria_client.clear_project_hierarchy(child_project_guid, parent_project_guid,body)

                    logger.success(f"===> Detached segment with {label} from `{child_project_guid}`to {parent_project_guid}\n")
                    out = parsed_output['display'].replace('Unlink', 'Link', 1)

                    return (out)


            elif object_action == "Link":
                if valid is False and exists:
                    msg = (f"-->  Link called `{label}` already exists and result document updated changing "
                           f"`Link` to `Detach` in processed output\n")
                    logger.error(msg)

                elif valid is False:
                    msg = f"==>{object_type} Link with label `{label}` is not valid and can't be created"
                    logger.error(msg)
                    return

                else:
                    body = set_rel_request_body_for_type("ProjectHierarchy", attributes)

                    egeria_client.set_project_hierarchy(project_guid =child_project_guid,
                                                        parent_project_guid = parent_project_guid)
                                                        # body=body_slimmer(body))
                    msg = f"==>Created {object_type} link named `{label}`\n"
                    logger.success(msg)
                    out = parsed_output['display'].replace('Link', 'Detach', 1)
                    return out

        except ValidationError as e:
            print_validation_error(e)
            logger.error(f"Validation Error performing {command}: {e}")
            return None
        except PyegeriaException as e:
            print_basic_exception(e)
            logger.error(f"PyegeriaException occurred: {e}")
            return None

        except Exception as e:
            logger.error(f"Error performing {command}: {e}")
            return None
    else:
        return None



def process_link_project_dependency_command(egeria_client: EgeriaTech, txt: str, directive: str = "display") -> Optional[str]:

#     """ Set one project dependence on another."""
#

    command, object_type, object_action = extract_command_plus(txt)
    print(Markdown(f"# {command}\n"))

    parsed_output = parse_view_command(egeria_client, object_type, object_action,
                                       txt, directive)

    if not parsed_output:
        logger.error(f"No output for `{object_action}`")
        print(Markdown("## Parsing failed"))
        return None

    print(Markdown(parsed_output['display']))

    logger.debug(json.dumps(parsed_output, indent=4))

    attributes = parsed_output['attributes']

    parent_project_guid = attributes.get('Parent Project', {}).get('guid', None)
    child_project_guid = attributes.get('Child Project', {}).get('guid', None)
    label = attributes.get('Link Label', {}).get('value', "")

    valid = parsed_output['valid']
    exists = parent_project_guid is not None and child_project_guid  is not None

    if directive == "display":

        return None
    elif directive == "validate":
        if valid:
            print(Markdown(f"==> Validation of {command} completed successfully!\n"))
        else:
            msg = f"Validation failed for object_action `{command}`\n"
            logger.error(msg)
            print(Markdown(f"==> Validation of {command} failed!!\n"))
        return valid

    elif directive == "process":


        try:
            if object_action == "Detach":
                if not exists:
                    msg = (f" Link `{label}` does not exist! Updating result document with Link "
                           f"object_action\n")
                    logger.error(msg)
                    out = parsed_output['display'].replace('Link', 'Detach', 1)
                    return out
                elif not valid:
                    return None
                else:
                    print(Markdown(
                        f"==> Validation of {command} completed successfully! Proceeding to apply the changes.\n"))
                    body = set_delete_request_body(object_type, attributes)

                    egeria_client.clear_project_dependency(child_project_guid, parent_project_guid,body)

                    logger.success(f"===> Detached segment with {label} from `{child_project_guid}`to {parent_project_guid}\n")
                    out = parsed_output['display'].replace('Unlink', 'Link', 1)

                    return (out)


            elif object_action == "Link":
                if valid is False and exists:
                    msg = (f"-->  Link called `{label}` already exists and result document updated changing "
                           f"`Link` to `Detach` in processed output\n")
                    logger.error(msg)

                elif valid is False:
                    msg = f"==>{object_type} Link with label `{label}` is not valid and can't be created"
                    logger.error(msg)
                    return

                else:
                    body = set_rel_request_body_for_type("ProjectDependency", attributes)

                    egeria_client.set_project_dependency(project_guid =child_project_guid,
                                                        upstream_project_guid = parent_project_guid,
                                                        body=body_slimmer(body))
                    msg = f"==>Created {object_type} link named `{label}`\n"
                    logger.success(msg)
                    out = parsed_output['display'].replace('Link', 'Detach', 1)
                    return out

        except ValidationError as e:
            print_validation_error(e)
            logger.error(f"Validation Error performing {command}: {e}")
            return None
        except PyegeriaException as e:
            print_basic_exception(e)
            logger.error(f"PyegeriaException occurred: {e}")
            return None

        except Exception as e:
            logger.error(f"Error performing {command}: {e}")
            return None
    else:
        return None



"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.



Execute Generic actions.

"""

import csv
import json
import os
import sys
import time
from datetime import datetime

import click


from pyegeria import EgeriaTech, body_slimmer
from pyegeria._exceptions import (
    InvalidParameterException,
    PropertyServerException,
    print_exception_response,
)

EGERIA_METADATA_STORE = os.environ.get("EGERIA_METADATA_STORE", "active-metadata-store")
EGERIA_KAFKA_ENDPOINT = os.environ.get("KAFKA_ENDPOINT", "localhost:9092")
EGERIA_PLATFORM_URL = os.environ.get("EGERIA_PLATFORM_URL", "https://localhost:9443")
EGERIA_VIEW_SERVER = os.environ.get("EGERIA_VIEW_SERVER", "view-server")
EGERIA_VIEW_SERVER_URL = os.environ.get(
    "EGERIA_VIEW_SERVER_URL", "https://localhost:9443"
)
EGERIA_INTEGRATION_DAEMON = os.environ.get("EGERIA_INTEGRATION_DAEMON", "integration-daemon")
EGERIA_INTEGRATION_DAEMON_URL = os.environ.get(
    "EGERIA_INTEGRATION_DAEMON_URL", "https://localhost:9443"
)
EGERIA_ADMIN_USER = os.environ.get("ADMIN_USER", "garygeeke")
EGERIA_ADMIN_PASSWORD = os.environ.get("ADMIN_PASSWORD", "secret")
EGERIA_USER = os.environ.get("EGERIA_USER", "erinoverview")
EGERIA_USER_PASSWORD = os.environ.get("EGERIA_USER_PASSWORD", "secret")
EGERIA_WIDTH = os.environ.get("EGERIA_WIDTH", 200)
EGERIA_JUPYTER = os.environ.get("EGERIA_JUPYTER", False)
EGERIA_HOME_GLOSSARY_GUID = os.environ.get("EGERIA_HOME_GLOSSARY_GUID", None)
EGERIA_GLOSSARY_PATH = os.environ.get("EGERIA_GLOSSARY_PATH", None)



@click.command("delete-element")
@click.option("--cascade", default = True, help = "If True, cascade the delete", is_flag=True)
@click.option("--server", default=EGERIA_VIEW_SERVER, help="Egeria view server to use")
@click.option(
    "--url", default=EGERIA_VIEW_SERVER_URL, help="URL of Egeria platform to connect to"
)
@click.option("--userid", default=EGERIA_USER, help="Egeria user")
@click.option("--password", default=EGERIA_USER_PASSWORD, help="Egeria user password")
@click.option("--timeout", default=60, help="Number of seconds to wait")
@click.argument("element-guid")
def delete_element(cascade, server, url, userid, password, timeout, element_guid):
    """Delete the glossary specified"""
    m_client = EgeriaTech(server, url, user_id=userid, user_pwd=password)
    token = m_client.create_egeria_bearer_token()
    try:
        m_client.delete_metadata_element_in_store(element_guid, cascade = cascade)

        click.echo(f"Deleted element: {element_guid}")

    except (InvalidParameterException, PropertyServerException) as e:
        print_exception_response(e)
    finally:
        m_client.close_session()





"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.



Egeria Coco Pharmaceutical demonstration labs.

This script creates and configures the monitorDev01 integration daemon.

The monitorDev01 is another integration daemon server supporting the development team.
It is cataloguing assets in preparation for deployment into production.
It has no integration services active initially since they are set up in the labs and demos that use it.

"""

from globals import adminUserId, devPlatformURL, max_paging_size

from pyegeria import print_exception_response
from pyegeria.full_omag_server_config import FullServerConfig

daemon_server_name = "monitorDev01"
daemon_server_platform = devPlatformURL
daemon_server_user_id = "erinoverview"
daemon_server_password = "erinoverviewpassw0rd"

mdr_server = "cocoMDSx"
platform_url = devPlatformURL

print("Configuring " + daemon_server_name + "...")

try:
    f_client = FullServerConfig(daemon_server_name, daemon_server_platform, adminUserId)

    f_client.set_basic_server_properties(
        "Integration daemon supporting the development team",
        "Coco Pharmaceuticals",
        daemon_server_platform,
        daemon_server_user_id,
        daemon_server_password,
        max_paging_size,
    )

    security_connection_body = {
        "class": "Connection",
        "connectorType": {
            "class": "ConnectorType",
            "connectorProviderClassName": "org.odpi.openmetadata.metadatasecurity.samples.CocoPharmaServerSecurityProvider",
        },
    }
    f_client.set_server_security_connection(security_connection_body)
    f_client.add_default_log_destinations()

    print(
        f"Initial configuration for {daemon_server_name} is complete - more configuration and activation "
        f"will be performed in the labs"
    )

except Exception as e:
    print_exception_response(e)



from pyegeria.output_formatter import generate_entity_dict


def fake_extract(element: dict, columns_struct: dict | None = None):
    """Populate provided columns_struct values from element by key.
    This mimics the extract_properties_func contract used by generate_entity_dict
    when a columns_struct is provided.
    """
    if columns_struct is None:
        return element
    fmt = columns_struct.get('formats', {})
    cols = fmt.get('attributes', [])
    for col in cols:
        key = col.get('key')
        if key is None:
            continue
        # simulate populate_common_columns header overlay behavior
        # prefer explicit element key (already set on element), fallback to properties
        col['value'] = element.get(key)
    return columns_struct


def test_generate_entity_dict_uses_per_element_copy_to_avoid_leakage():
    # Two elements with distinct guids and qualified names
    e1 = {
        'elementHeader': {'guid': 'guid-1'},
        'properties': {'qualifiedName': 'Q1', 'displayName': 'D1'},
        'guid': 'guid-1',
        'qualified_name': 'Q1',
        'type_name': 'TypeA',
    }
    e2 = {
        'elementHeader': {'guid': 'guid-2'},
        'properties': {'qualifiedName': 'Q2', 'displayName': 'D2'},
        'guid': 'guid-2',
        'qualified_name': 'Q2',
        'type_name': 'TypeB',
    }

    columns_struct = {
        'formats': {
            'attributes': [
                {'name': 'Qualified Name', 'key': 'qualified_name'},
                {'name': 'GUID', 'key': 'guid'},
                {'name': 'Type Name', 'key': 'type_name'},
            ]
        }
    }

    rows = generate_entity_dict(
        elements=[e1, e2],
        extract_properties_func=fake_extract,
        columns_struct=columns_struct,
        output_format='DICT',
    )

    assert isinstance(rows, list) and len(rows) == 2

    r1, r2 = rows
    # Ensure no value leakage: each row's GUID and Qualified Name correspond to its own element
    assert r1['GUID'] == 'guid-1'
    assert r1['Qualified Name'] == 'Q1'
    assert r1['Type Name'] == 'TypeA'

    assert r2['GUID'] == 'guid-2'
    assert r2['Qualified Name'] == 'Q2'
    assert r2['Type Name'] == 'TypeB'



# Auto-generated by gen_format_sets.py
from pyegeria._output_format_models import Column, Format, ActionParameter, FormatSet, FormatSetDict

generated_format_sets = FormatSetDict({
    'Agreement-DrE': FormatSet(target_type='Agreement', heading='Agreement-DrE Attributes', description='Auto-generated format for Agreement (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Identifier', key='agreement_identifier'), Column(name='Category', key='category'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Version Identifier', key='version_identifier'), Column(name='Agreement Actors', key='agreement_actors'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Business-Imperative-DrE': FormatSet(target_type='Business Imperative', heading='Business-Imperative-DrE Attributes', description='Auto-generated format for Business Imperative (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Category-DrE': FormatSet(target_type='Category', heading='Category-DrE Attributes', description='Auto-generated format for Category (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Description', key='description'), Column(name='In Glossary', key='in_glossary'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'certification_type-DrE': FormatSet(target_type='certification_type', heading='certification_type-DrE Attributes', description='Auto-generated format for certification_type (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Document Identifier', key='document_identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Details', key='details'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Collection-DrE': FormatSet(target_type='Collection', heading='Collection-DrE Attributes', description='Auto-generated format for Collection (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Classifications', key='classifications'), Column(name='Category', key='category'), Column(name='Version Identifier', key='current_version'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Data-Class-DrE': FormatSet(target_type='Data Class', heading='Data-Class-DrE Attributes', description='Auto-generated format for Data Class (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Status', key='element_status'), Column(name='Namespace', key='namespace'), Column(name='Match Property Names', key='match_property_names'), Column(name='Match Threshold', key='match_threshold'), Column(name='IsCaseSensitive', key='is_case_sensitive'), Column(name='Data Type', key='data_type'), Column(name='Allow Duplicate Values', key='duplicates_allowed'), Column(name='isNullable', key='is_nullable'), Column(name='Default Value', key='default_value'), Column(name='Average Value', key='avg_value'), Column(name='Value List', key='value_list'), Column(name='Value Range From', key='value_range_from'), Column(name='Value Range To', key='value_range_to'), Column(name='Sample Values', key='sample_values'), Column(name='Data Patterns', key='data_patterns'), Column(name='In Data Dictionary', key='in_data_dictionary'), Column(name='Containing Data Class', key='containing_data_class'), Column(name='Specializes Data Class', key='specializes_data_class'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Data-Dictionary-DrE': FormatSet(target_type='Data Dictionary', heading='Data-Dictionary-DrE Attributes', description='Auto-generated format for Data Dictionary (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Data-Field-DrE': FormatSet(target_type='Data Field', heading='Data-Field-DrE Attributes', description='Auto-generated format for Data Field (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Status', key='element_status'), Column(name='Data Type', key='data_type'), Column(name='Position', key='position'), Column(name='Minimum Cardinality', key='min_cardinality'), Column(name='Maximum Cardinality', key='max_cardinality'), Column(name='In Data Structure', key='in_data_structure'), Column(name='Data Class', key='data_class'), Column(name='Glossary Term', key='glossary_term'), Column(name='isNullable', key='is_nullable'), Column(name='Minimum Length', key='min_length'), Column(name='Length', key='length'), Column(name='Precision', key='precision'), Column(name='Ordered Values', key='ordered_values'), Column(name='Units', key='units'), Column(name='Default Value', key='default_value'), Column(name='Version Identifier', key='version_id'), Column(name='In Data Dictionary', key='in_data_dictionary'), Column(name='Parent Data Field', key='parent_data_field'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Data-Processing-Purpose-DrE': FormatSet(target_type='Data Processing Purpose', heading='Data-Processing-Purpose-DrE Attributes', description='Auto-generated format for Data Processing Purpose (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Data-Sharing-Agreement-DrE': FormatSet(target_type='Data Sharing Agreement', heading='Data-Sharing-Agreement-DrE Attributes', description='Auto-generated format for Data Sharing Agreement (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Identifier', key='identifier'), Column(name='Status', key='element_status'), Column(name='User_Defined_Status', key='user_defined_status'), Column(name='Version Identifier', key='version_identifier'), Column(name='Product Manager', key='product_manager'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Data-Specification-DrE': FormatSet(target_type='Data Specification', heading='Data-Specification-DrE Attributes', description='Auto-generated format for Data Specification (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Collection Type', key='collection_type'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Data-Structure-DrE': FormatSet(target_type='Data Structure', heading='Data-Structure-DrE Attributes', description='Auto-generated format for Data Structure (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Status', key='element_status'), Column(name='In Data Specification', key='in_data_spec'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Digital-Product-Catalog-DrE': FormatSet(target_type='Digital Product Catalog', heading='Digital-Product-Catalog-DrE Attributes', description='Auto-generated format for Digital Product Catalog (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Classifications', key='classifications'), Column(name='Category', key='category'), Column(name='Version Identifier', key='current_version'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Digital-Product-DrE': FormatSet(target_type='Digital Product', heading='Digital-Product-DrE Attributes', description='Auto-generated format for Digital Product (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Product Name', key='product_name'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Category', key='category'), Column(name='Identifier', key='identifier'), Column(name='Maturity', key='maturity'), Column(name='Service Life', key='service_life'), Column(name='Introduction Date', key='introduction_date'), Column(name='Next Version Date', key='next_version_date'), Column(name='Withdrawal Date', key='withdrawal_date'), Column(name='Version Identifier', key='current_version'), Column(name='Product Manager', key='product_manager'), Column(name='Agreements', key='agreements'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Digital-Subscription-DrE': FormatSet(target_type='Digital Subscription', heading='Digital-Subscription-DrE Attributes', description='Auto-generated format for Digital Subscription (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Identifier', key='identifier'), Column(name='Category', key='category'), Column(name='Status', key='element_status'), Column(name='User_Defined_Status', key='user_defined_status'), Column(name='Support Level', key='support_level'), Column(name='Service Levels', key='service_levels'), Column(name='Version Identifier', key='version_identifier'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Glossary-DrE': FormatSet(target_type='Glossary', heading='Glossary-DrE Attributes', description='Auto-generated format for Glossary (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Language', key='language'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Governance-Action-DrE': FormatSet(target_type='Governance Action', heading='Governance-Action-DrE Attributes', description='Auto-generated format for Governance Action (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Governance-Approach-DrE': FormatSet(target_type='Governance Approach', heading='Governance-Approach-DrE Attributes', description='Auto-generated format for Governance Approach (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Governance-Control-DrE': FormatSet(target_type='Governance Control', heading='Governance-Control-DrE Attributes', description='Auto-generated format for Governance Control (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='measurement', key='measurement'), Column(name='target', key='target'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Governance-Driver-DrE': FormatSet(target_type='Governance Driver', heading='Governance-Driver-DrE Attributes', description='Auto-generated format for Governance Driver (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Governance-Obligation-DrE': FormatSet(target_type='Governance Obligation', heading='Governance-Obligation-DrE Attributes', description='Auto-generated format for Governance Obligation (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Governance-Policy-DrE': FormatSet(target_type='Governance Policy', heading='Governance-Policy-DrE Attributes', description='Auto-generated format for Governance Policy (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Governance-Principle-DrE': FormatSet(target_type='Governance Principle', heading='Governance-Principle-DrE Attributes', description='Auto-generated format for Governance Principle (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'governance_procedure-DrE': FormatSet(target_type='governance_procedure', heading='governance_procedure-DrE Attributes', description='Auto-generated format for governance_procedure (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Document Identifier', key='document_identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implementation Description', key='implementation_description'), Column(name='Supports Policies', key='supports_policies'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'governance_responsibility-DrE': FormatSet(target_type='governance_responsibility', heading='governance_responsibility-DrE Attributes', description='Auto-generated format for governance_responsibility (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Document Identifier', key='document_identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implementation Description', key='implementation_description'), Column(name='Supports Policies', key='supports_policies'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'governance_rule-DrE': FormatSet(target_type='governance_rule', heading='governance_rule-DrE Attributes', description='Auto-generated format for governance_rule (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'governance_strategy-DrE': FormatSet(target_type='governance_strategy', heading='governance_strategy-DrE Attributes', description='Auto-generated format for governance_strategy (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Information-Supply-Chain-DrE': FormatSet(target_type='Information Supply Chain', heading='Information-Supply-Chain-DrE Attributes', description='Auto-generated format for Information Supply Chain (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Scope', key='scope'), Column(name='Purposes', key='purposes'), Column(name='Nested Information Supply Chains', key='nested_info_supply_chains'), Column(name='In Information Supply Chain', key='in_supply_chain'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid'), Column(name='Merge Update', key='merge_update')])]),
    'license_type-DrE': FormatSet(target_type='license_type', heading='license_type-DrE Attributes', description='Auto-generated format for license_type (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Document Identifier', key='document_identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Details', key='details'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'naming_standard_rule-DrE': FormatSet(target_type='naming_standard_rule', heading='naming_standard_rule-DrE Attributes', description='Auto-generated format for naming_standard_rule (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Document Identifier', key='document_identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implementation Description', key='implementation_description'), Column(name='Supports Policies', key='supports_policies'), Column(name='Name Patterns', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Project-DrE': FormatSet(target_type='Project', heading='Project-DrE Attributes', description='Auto-generated format for Project (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Project Type', key='project_type'), Column(name='Category', key='category'), Column(name='Identifier', key='project_identifier'), Column(name='Mission', key='mission'), Column(name='Purposes', key='purposes'), Column(name='Start Date', key='start_date'), Column(name='Planned End Date', key='end_date'), Column(name='Priority', key='priority'), Column(name='Project Phase', key='project_phase'), Column(name='Project Status', key='project_status'), Column(name='Project Health', key='project_health'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Regulation-Article-DrE': FormatSet(target_type='Regulation Article', heading='Regulation-Article-DrE Attributes', description='Auto-generated format for Regulation Article (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'regulation-DrE': FormatSet(target_type='regulation', heading='regulation-DrE Attributes', description='Auto-generated format for regulation (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Regulation Source', key='regulation_source'), Column(name='Regulators', key='regulators'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'security_access_control-DrE': FormatSet(target_type='security_access_control', heading='security_access_control-DrE Attributes', description='Auto-generated format for security_access_control (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='criteria', key='criteria'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'security_group-DrE': FormatSet(target_type='security_group', heading='security_group-DrE Attributes', description='Auto-generated format for security_group (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'service_level_objective-DrE': FormatSet(target_type='service_level_objective', heading='service_level_objective-DrE Attributes', description='Auto-generated format for service_level_objective (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Solution-Blueprint-DrE': FormatSet(target_type='Solution Blueprint', heading='Solution-Blueprint-DrE Attributes', description='Auto-generated format for Solution Blueprint (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Status', key='element_status'), Column(name='Version Identifier', key='version_id'), Column(name='Solution Components', key='solution_components'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Solution-Component-DrE': FormatSet(target_type='Solution Component', heading='Solution-Component-DrE Attributes', description='Auto-generated format for Solution Component (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Qualified Name', key='qualified_name'), Column(name='Category', key='category'), Column(name='Description', key='description'), Column(name='Status', key='element_status'), Column(name='Solution Component Type', key='soln_comp_type'), Column(name='Planned Deployed Implementation Type', key='planned_deployed_impl_type'), Column(name='Initial Status', key='initial_status'), Column(name='In Solution Components', key='in_components'), Column(name='In Solution Blueprints', key='solution_blueprints'), Column(name='In Information Supply Chains', key='in_supply_chains'), Column(name='Actors', key='actors'), Column(name='GUID', key='guid'), Column(name='Merge Update', key='merge_update')])]),
    'Solution-Role-DrE': FormatSet(target_type='Solution Role', heading='Solution-Role-DrE Attributes', description='Auto-generated format for Solution Role (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Name', key='name'), Column(name='Description', key='description'), Column(name='Title', key='title'), Column(name='Scope', key='scope'), Column(name='identifier', key='identifier'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Category', key='category'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Threat-Definition-DrE': FormatSet(target_type='Threat Definition', heading='Threat-Definition-DrE Attributes', description='Auto-generated format for Threat Definition (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])])
})



#!/usr/bin/env python3
"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

Unit tests for the Utils helper functions using the Pytest framework.


A simple display for glossary terms
"""
import argparse
import os
import sys
import time

from rich import box
from rich.console import Console
from rich.markdown import Markdown
from rich.prompt import Prompt
from rich.table import Table
from rich.text import Text

from pyegeria import (
    EgeriaTech,
    InvalidParameterException,
    PropertyServerException,
    UserNotAuthorizedException,
    print_exception_response,
)

disable_ssl_warnings = True

EGERIA_METADATA_STORE = os.environ.get("EGERIA_METADATA_STORE", "active-metadata-store")
EGERIA_KAFKA_ENDPOINT = os.environ.get("KAFKA_ENDPOINT", "localhost:9092")
EGERIA_PLATFORM_URL = os.environ.get("EGERIA_PLATFORM_URL", "https://localhost:9443")
EGERIA_VIEW_SERVER = os.environ.get("EGERIA_VIEW_SERVER", "view-server")
EGERIA_VIEW_SERVER_URL = os.environ.get(
    "EGERIA_VIEW_SERVER_URL", "https://localhost:9443"
)
EGERIA_INTEGRATION_DAEMON = os.environ.get("EGERIA_INTEGRATION_DAEMON", "integration-daemon")
EGERIA_ADMIN_USER = os.environ.get("ADMIN_USER", "garygeeke")
EGERIA_ADMIN_PASSWORD = os.environ.get("ADMIN_PASSWORD", "secret")
EGERIA_USER = os.environ.get("EGERIA_USER", "erinoverview")
EGERIA_USER_PASSWORD = os.environ.get("EGERIA_USER_PASSWORD", "secret")
EGERIA_JUPYTER = bool(os.environ.get("EGERIA_JUPYTER", "False"))
EGERIA_WIDTH = int(os.environ.get("EGERIA_WIDTH", "200"))
EGERIA_GLOSSARY_PATH = os.environ.get("EGERIA_GLOSSARY_PATH", None)
EGERIA_ROOT_PATH = os.environ.get("EGERIA_ROOT_PATH", "../../")
EGERIA_INBOX_PATH = os.environ.get("EGERIA_INBOX_PATH", "md_processing/dr_egeria_inbox")
EGERIA_OUTBOX_PATH = os.environ.get("EGERIA_OUTBOX_PATH", "md_processing/dr_egeria_outbox")



def display_glossaries(
    search_string: str = "*",
    view_server: str = EGERIA_VIEW_SERVER,
    view_url: str = EGERIA_VIEW_SERVER_URL,
    user: str = EGERIA_USER,
    user_pass: str = EGERIA_USER_PASSWORD,
    jupyter: bool = EGERIA_JUPYTER,
    width: int = EGERIA_WIDTH,
    output_format: str = "JSON"
):
    """Display either a specified glossary or all glossaries if the search_string is '*'.
    Parameters
    ----------
    search_string : str, default is '*'
        The string used to search for glossaries.
    view_server : str
        The view server name or address where the Egeria services are hosted.
    view_url : str
        The URL of the platform the view server is on.
    user : str
        The user ID for authentication with the Egeria server.
    user_pass : str
        The password for authentication with the Egeria server.
    jupyter : bool, optional
        A boolean indicating whether the output is intended for a Jupyter notebook (default is EGERIA_JUPYTER).
    width : int, optional
        The width of the console output (default is EGERIA_WIDTH).
    output_format: str, optional, default is 'JSON'
        One of TABLE, FORM, REPORT
    """
    m_client = EgeriaTech(view_server, view_url, user_id=user, user_pwd=user_pass)
    token = m_client.create_egeria_bearer_token()
    console = Console(
        style="bold bright_white on black",
        width=width,
        force_terminal=not jupyter,
        )

    try:
        if output_format == "FORM":
            action = "Update-Form"
        elif output_format == "REPORT":
            action = "Report"
        if output_format != "TABLE":
            file_path = os.path.join(EGERIA_ROOT_PATH, EGERIA_OUTBOX_PATH)
            file_name = f"Glossaries-{time.strftime('%Y-%m-%d-%H-%M-%S')}-{action}.md"
            full_file_path = os.path.join(file_path, file_name)
            os.makedirs(os.path.dirname(full_file_path), exist_ok=True)
            output = m_client.find_glossaries(search_string,  None, output_format=output_format)
            if output == "NO_GLOSSARIES_FOUND":
                print(f"\n==> No glossaries found for search string '{search_string}'")
                return
            try:
                with open(full_file_path, 'w') as f:
                    f.write(output)
                print(f"\n==> Glossaries output written to {full_file_path}")
                return
            except Exception:
                console.print_exception()

        table = Table(
            title=f"Glossary List @ {time.asctime()}",
            style="bright_white on black",
            header_style="bright_white on dark_blue",
            title_style="bold white on black",
            caption_style="white on black",
            show_lines=True,
            box=box.ROUNDED,
            caption=f"View Server '{view_server}' @ Platform - {view_url}",
            expand=True,
        )
        table.add_column("Glossary Name")
        table.add_column(
            "Qualified Name & GUID", width=38, no_wrap=True, justify="center"
        )
        table.add_column("Language")
        table.add_column("Description")
        table.add_column("Usage")
        table.add_column("Categories")

        glossaries = m_client.find_glossaries(search_string)
        if type(glossaries) is list:
            sorted_glossary_list = sorted(
                glossaries, key=lambda k: k["properties"]["displayName"]
            )
            for glossary in sorted_glossary_list:
                display_name = glossary["properties"].get("displayName",'---')
                qualified_name = glossary["properties"]["qualifiedName"]
                guid = glossary["elementHeader"]["guid"]
                q_name = Text(f"{qualified_name}\n&\n{guid}", justify="center")
                language = glossary["properties"].get("language",'---')
                description = glossary["properties"].get("description",'---')
                usage = glossary["properties"].get("usage",'---')

                members = glossary.get("collectionMembers", [])
                cat_md = ''
                if type(members) is list:
                    for member in members:
                        if member['relatedElement'
                        ]["elementHeader"]["type"]['typeName'] == "CollectionFolder":
                            cat_md += f'* {member.get("relatedElement",{}).get("properties",{}).get("qualifiedName","")}\n'
                    cat_md = cat_md.strip()

                table.add_row(display_name, q_name, language, description, usage, Markdown(cat_md))

            console.print(table)

    except (InvalidParameterException, PropertyServerException) as e:
        print_exception_response(e)
    finally:
        m_client.close_session()


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--server", help="Name of the server to display status for")
    parser.add_argument("--url", help="URL Platform to connect to")
    parser.add_argument("--userid", help="User Id")
    parser.add_argument("--password", help="User Password")

    args = parser.parse_args()

    server = args.server if args.server is not None else EGERIA_VIEW_SERVER
    url = args.url if args.url is not None else EGERIA_PLATFORM_URL
    userid = args.userid if args.userid is not None else EGERIA_USER
    user_pass = args.password if args.password is not None else EGERIA_USER_PASSWORD

    try:
        search_string = Prompt.ask(
            "Enter the glossary you are searching for or '*' for all:", default="*"
        )
        output_format = Prompt.ask("What output format do you want?", choices=["TABLE", "FORM", "REPORT"], default="TABLE")
        display_glossaries(search_string, server, url, userid,
                           user_pass, output_format=output_format)

    except KeyboardInterrupt:
        pass


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

Unit tests for the Utils helper functions using the Pytest framework.


A simple status display for Governance Actions
"""
import argparse
import os
import time

from rich import box
from rich.console import Console
from rich.live import Live
from rich.markdown import Markdown
from rich.prompt import Prompt
from rich.table import Table

from pyegeria import (
    EgeriaTech,
    InvalidParameterException,
    PropertyServerException,
    UserNotAuthorizedException,
)

EGERIA_METADATA_STORE = os.environ.get("EGERIA_METADATA_STORE", "active-metadata-store")
EGERIA_KAFKA_ENDPOINT = os.environ.get("KAFKA_ENDPOINT", "localhost:9092")
EGERIA_PLATFORM_URL = os.environ.get("EGERIA_PLATFORM_URL", "https://localhost:9443")
EGERIA_VIEW_SERVER = os.environ.get("EGERIA_VIEW_SERVER", "view-server")
EGERIA_VIEW_SERVER_URL = os.environ.get(
    "EGERIA_VIEW_SERVER_URL", "https://localhost:9443"
)
EGERIA_INTEGRATION_DAEMON = os.environ.get(
    "EGERIA_INTEGRATION_DAEMON", "integration_daemon"
)
EGERIA_ENGINE_HOST = os.environ.get("EGERIA_ENGINE_HOST", "engine-host")
EGERIA_ENGINE_HOST_URL = os.environ.get(
    "EGERIA_ENGINE_HOST_URL", "https://localhost:9443"
)
EGERIA_ADMIN_USER = os.environ.get("EGERIA_ADMIN_USER", "garygeeke")
EGERIA_ADMIN_PASSWORD = os.environ.get("EGERIA_ADMIN_PASSWORD", "secret")
EGERIA_USER = os.environ.get("EGERIA_USER", "erinoverview")
EGERIA_USER_PASSWORD = os.environ.get("EGERIA_USER_PASSWORD", "secret")
EGERIA_JUPYTER = bool(os.environ.get("EGERIA_JUPYTER", "False"))
EGERIA_WIDTH = int(os.environ.get("EGERIA_WIDTH", "200"))


disable_ssl_warnings = True


def display_gov_eng_status(
    search_list: list[str] = ["*"],
    status_filter=["*"],
    engine_host: str = EGERIA_ENGINE_HOST,
    view_server: str = EGERIA_VIEW_SERVER,
    url: str = EGERIA_VIEW_SERVER_URL,
    username: str = EGERIA_USER,
    user_pass: str = EGERIA_USER_PASSWORD,
    paging: bool = True,
    jupyter: bool = EGERIA_JUPYTER,
    width: int = EGERIA_WIDTH,
    sort: bool = True,
) -> None:
    """Displays the status table of the governance engines on the specified Engine Host OMAG Server

    Parameters
    ----------

    search_list : list of str
        List of governance engine names to search for. Defaults to ["*"] which returns all governance engines.
    status_filter: list of str, default is all
        List of status values to filter by.
    engine_host : str
        The host name of the governance engine.
    view_server : str
        The name of the view server to interact with.
    url : str
        The URL of the view server.
    username : str
        Username for authentication with the view server.
    user_pass : str
        Password for authentication with the view server.
    paging : bool
        Determines whether to use a pager or live monitor for output. Defaults to True.
    jupyter : bool
        Specifies if the code is running in a Jupyter environment. Defaults to EGERIA_JUPYTER.
    width : int
        Width of the console output. Defaults to EGERIA_WIDTH.
    sort : bool
        Determines whether to sort the governance engine statuses. Defaults to True.

    Returns
    -------

    Nothing
    """
    console = Console(width=EGERIA_WIDTH)

    s_client = EgeriaTech(view_server, url, username, user_pass)
    token = s_client.create_egeria_bearer_token()

    def generate_table(search_list: list[str]) -> Table:
        """Make a new table."""
        table = Table(
            title=f"Governance Engine Status @ {time.asctime()}",
            style="bold white on black",
            row_styles=["bold white on black"],
            header_style="white on dark_blue",
            title_style="bold white on black",
            caption_style="white on black",
            show_lines=True,
            box=box.ROUNDED,
            caption=f"Server: '{engine_host}' running on {url}",
            expand=True,
        )
        # table.footer: f"Server {view_server} on Platform {good_platform1_url}"
        table.add_column("Gov Engine")
        table.add_column("Gov Engine Type")
        table.add_column("Gov Engine Desc")
        table.add_column("Engine Status")
        table.add_column("Request Types")

        # eng_host_guid = s_client.get_guid_for_name(engine_host)
        gov_eng_status = s_client.get_server_report(
            server_guid=None, server_name=engine_host
        )
        eng_summaries = gov_eng_status["governanceEngineSummaries"]

        if sort is True:
            engine_info = sorted(
                eng_summaries, key=lambda k: k.get("governanceEngineName", " ")
            )
        else:
            engine_info = eng_summaries

        for engine in engine_info:
            gov_eng = engine["governanceEngineName"]

            if (gov_eng not in search_list) and (search_list != ["*"]):
                # if specific engines are requested and it doesn't match, then skip
                continue
            eng_type = engine.get("governanceEngineTypeName", " ")

            eng_desc = engine.get("governanceEngineDescription", " ")
            eng_req_types = engine.get("governanceRequestTypes", " ")
            eng_req_type_md = " "
            if type(eng_req_types) is list:
                for i in eng_req_types:
                    eng_req_type_md += f"* {i}\n"
                eng_req_type_out = Markdown(eng_req_type_md)
            else:
                eng_req_type_out = " "
            status = engine["governanceEngineStatus"]
            if (status not in status_filter) and (status_filter != ["*"]):
                continue
            if status in ("RUNNING"):
                eng_status = f"[green]{status}"
            elif status in ("FAILED"):
                eng_status = f"[red]{status}"
            else:
                eng_status = f"[yellow]{status}"

            table.add_row(gov_eng, eng_type, eng_desc, eng_status, eng_req_type_out)

        token = s_client.create_egeria_bearer_token()
        return table

    try:
        if paging is True:
            console = Console(width=width, force_terminal=not jupyter)
            with console.pager():
                console.print(generate_table(search_list))
        else:
            with Live(
                generate_table(search_list),
                refresh_per_second=1,
                screen=True,
                vertical_overflow="visible",
            ) as live:
                while True:
                    time.sleep(2)
                    live.update(generate_table(search_list))

    except (
        InvalidParameterException,
        PropertyServerException,
        UserNotAuthorizedException,
    ) as e:
        console.print_exception(show_locals=True)

    except KeyboardInterrupt:
        pass

    finally:
        s_client.close_session()


def main_live():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--engine_host", help="Name of the server to display status for"
    )
    parser.add_argument("--view_server", help="Name of the view server to use")
    parser.add_argument("--url", help="URL Platform to connect to")
    parser.add_argument("--userid", help="User Id")
    parser.add_argument("--password", help="User Password")
    # parser.add_argument("--paging", help="Paging")
    args = parser.parse_args()

    engine_host = (
        args.engine_host if args.engine_host is not None else EGERIA_ENGINE_HOST
    )
    view_server = (
        args.view_server if args.view_server is not None else EGERIA_VIEW_SERVER
    )
    url = args.url if args.url is not None else EGERIA_ENGINE_HOST_URL
    userid = args.userid if args.userid is not None else EGERIA_USER
    user_pass = args.password if args.password is not None else EGERIA_USER_PASSWORD
    paging = False

    search_list = Prompt.ask(
        "Enter the list of engines you are interested in or ['*'] for all",
        default=["*"],
    )
    status_filter = Prompt.ask(
        "Enter a list of status values to filter for or ['*'] for all",
        default=["*"],
    )
    display_gov_eng_status(
        search_list,
        engine_host=engine_host,
        view_server=view_server,
        url=url,
        username=userid,
        user_pass=user_pass,
        paging=paging,
        status_filter=status_filter,
    )


def main_paging():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--engine_host", help="Name of the engine host to display status for"
    )
    parser.add_argument("--view_server", help="Name of the view server to use")
    parser.add_argument("--url", help="URL of Platform to connect to")
    parser.add_argument("--userid", help="User Id")
    parser.add_argument("--password", help="User Password")
    args = parser.parse_args()

    engine_host = (
        args.engine_host if args.engine_host is not None else EGERIA_ENGINE_HOST
    )
    view_server = (
        args.view_server if args.view_server is not None else EGERIA_VIEW_SERVER
    )
    url = args.url if args.url is not None else EGERIA_VIEW_SERVER_URL
    userid = args.userid if args.userid is not None else EGERIA_USER
    user_pass = args.password if args.password is not None else EGERIA_USER_PASSWORD
    paging = True
    search_list = Prompt.ask(
        "Enter the list of engines you are interested in or ['*'] for all",
        default=["*"],
    )
    status_filter = Prompt.ask(
        "Enter a list of status values to filter for or ['*'] for all",
        default=["*"],
    )
    display_gov_eng_status(
        search_list,
        engine_host=engine_host,
        view_server=view_server,
        url=url,
        username=userid,
        user_pass=user_pass,
        paging=paging,
        status_filter=status_filter,
    )


if __name__ == "__main__":
    main_live()

if __name__ == "__main_paging__":
    main_paging()



#!/usr/bin/env python3
"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.


A object_action line interface for Egeria Users - of individual relevance

This is an emerging capability based on the **click** package. Feedback welcome!

"""
import os

import click
from trogon import tui

from commands.cat.my_reports import start_exp2
from commands.cli.ops_config import Config
from commands.my.list_my_profile import display_my_profile
from commands.my.list_my_roles import display_my_roles
from commands.my.monitor_my_todos import display_my_todos
from commands.my.monitor_open_todos import display_todos
from commands.my.todo_actions import (
    change_todo_status,
    create_todo,
    delete_todo,
    mark_todo_complete,
    reassign_todo,
)

# class Config(object):
#     def __init__(self, server: str = None, url: str = None, userid:str = None, password:str = None,
#                  timeout:int = 30, paging: bool = False):
#         self.server = server
#         self.url = url
#         self.userid = userid
#         self.password = password
#         self.timeout = timeout
#         self.paging = paging
#
#
# pass_config = click.make_pass_decorator(Config)


@tui()
# @tui('menu', 'menu', 'A textual object_action line interface')
@click.version_option("0.0.1", prog_name="egeria_ops")
@click.group()
@click.option(
    "--server",
    default=os.environ.get("EGERIA_METADATA_STORE", "qs-metadata-store"),
    help="Egeria metadata store to work with",
)
@click.option(
    "--url",
    default=os.environ.get("EGERIA_PLATFORM_URL", "https://localhost:9443"),
    help="URL of Egeria metadata store platform to connect to",
)
@click.option(
    "--integration_daemon",
    default=os.environ.get("EGERIA_INTEGRATION_DAEMON", "qs-integration-daemon"),
    help="Egeria integration daemon to work with",
)
@click.option(
    "--integration_daemon_url",
    default=os.environ.get("EGERIA_INTEGRATION_DAEMON_URL", "https://localhost:9443"),
    help="URL of Egeria integration daemon platform to connect to",
)
@click.option(
    "--view_server",
    default=os.environ.get("EGERIA_VIEW_SERVER", "qs-view-server"),
    help="Egeria view server to work with",
)
@click.option(
    "--view_server_url",
    default=os.environ.get("EGERIA_VIEW_SERVER_URL", "https://localhost:9443"),
    help="URL of Egeria view server platform to connect to",
)
@click.option(
    "--engine_host",
    default=os.environ.get("EGERIA_ENGINE_HOST", "qs-engine-host"),
    help="Egeria engine host to work with",
)
@click.option(
    "--engine_host_url",
    default=os.environ.get("EGERIA_ENGINE_HOST_URL", "https://localhost:9443"),
    help="URL of Egeria engine host platform to connect to",
)
@click.option(
    "--admin_user",
    default=os.environ.get("EGERIA_ADMIN_USER", "garygeeke"),
    help="Egeria admin user",
)
@click.option(
    "--admin_user_password",
    default=os.environ.get("EGERIA_ADMIN_PASSWORD", "secret"),
    help="Egeria admin password",
)
@click.option(
    "--userid",
    default=os.environ.get("EGERIA_USER", "peterprofile"),
    help="Egeria user",
)
@click.option(
    "--password",
    default=os.environ.get("EGERIA_USER_PASSWORD", "secret"),
    help="Egeria user password",
)
@click.option("--timeout", default=60, help="Number of seconds to wait")
@click.option(
    "--jupyter",
    is_flag=True,
    default=os.environ.get("EGERIA_JUPYTER", "False"),
    help="Enable for rendering in a Jupyter terminal",
)
@click.option(
    "--width",
    default=os.environ.get("EGERIA_WIDTH", "200"),
    help="Screen width, in characters, to use",
)
@click.option(
    "--home_glossary_guid",
    default=os.environ.get("EGERIA_HOME_GLOSSARY_GUID", None),
    help="Glossary guid to use as the home glossary",
)
@click.option(
    "--glossary_path",
    default=os.environ.get("EGERIA_GLOSSARY_PATH", "/home/jovyan/loading-bay/glossary"),
    help="Path to glossary import/export files",
)
@click.pass_context
def cli(
    ctx,
    server,
    url,
    view_server,
    view_server_url,
    integration_daemon,
    integration_daemon_url,
    engine_host,
    engine_host_url,
    admin_user,
    admin_user_password,
    userid,
    password,
    timeout,
    jupyter,
    width,
    home_glossary_guid,
    glossary_path,
):
    """An Egeria Command Line interface for Operations"""
    ctx.obj = Config(
        server,
        url,
        view_server,
        view_server_url,
        integration_daemon,
        integration_daemon_url,
        engine_host,
        engine_host_url,
        admin_user,
        admin_user_password,
        userid,
        password,
        timeout,
        jupyter,
        width,
        home_glossary_guid,
        glossary_path,
    )
    ctx.max_content_width = 200
    ctx.ensure_object(Config)


@cli.group("show")
@click.pass_context
def show(ctx):
    """Display an Egeria Object"""
    pass

@show.command("my-reports")
@click.pass_context
def show_my_reports(ctx):
    """Display my profiles

    Usage: show my-profile

    """
    c = ctx.obj
    start_exp2(
        # c.view_server, c.view_server_url, c.userid, c.password, c.jupyter, c.width
    )

@show.command("my-profile")
@click.pass_context
def show_my_profile(ctx):
    """Display my profiles

    Usage: show my-profile

    """
    c = ctx.obj
    display_my_profile(
        c.view_server, c.view_server_url, c.userid, c.password, c.jupyter, c.width
    )


@show.command("my-roles")
@click.pass_context
def show_my_roles(ctx):
    """Display my roles"""
    c = ctx.obj
    display_my_roles(
        c.view_server, c.view_server_url, c.userid, c.password, c.jupyter, c.width
    )


@show.command("my-to-dos")
@click.pass_context
def show_my_todos(ctx):
    """Show my To-Dos

    Usage: show my-to-dos

    """
    c = ctx.obj
    display_my_todos(
        c.view_server, c.view_server_url, c.userid, c.password, c.jupyter, c.width
    )


@show.command("open-to-dos")
@click.pass_context
def show_open_todos(ctx):
    """Display a live status view of Egeria servers for the specified Egeria platform

    Usage: show tech-details <tech-name>

           tech-name is a valid technology name (see 'show tech-types')
    """
    c = ctx.obj
    display_todos(
        c.view_server, c.view_server_url, c.userid, c.password, c.jupyter, c.width
    )


#
#  Tell
#


@cli.group("tell")
@click.pass_context
def tell(ctx):
    """Perform actions an Egeria Objects"""
    pass


tell.add_command(create_todo)
tell.add_command(delete_todo)
tell.add_command(change_todo_status)
tell.add_command(mark_todo_complete)
tell.add_command(reassign_todo)


if __name__ == "__main__":
    cli()



"""
This file contains general utility functions for processing Egeria Markdown
"""
import os
import sys
from typing import List

from loguru import logger
from rich import print
from rich.console import Console

from commands.cat.dr_egeria_command_help import EGERIA_INBOX_PATH
from md_processing.md_processing_utils.common_md_utils import (get_current_datetime_string, get_element_dictionary,
                                                               update_element_dictionary,
                                                               split_tb_string, str_to_bool, )
from md_processing.md_processing_utils.extraction_utils import (process_simple_attribute, extract_attribute,
                                                                get_element_by_name)
from md_processing.md_processing_utils.md_processing_constants import (get_command_spec, load_commands, load_commands, COMMAND_DEFINITIONS)

from md_processing.md_processing_utils.message_constants import (ERROR, INFO, WARNING, ALWAYS, EXISTS_REQUIRED)
from pyegeria import EgeriaTech
from pyegeria._globals import DEBUG_LEVEL


# Constants
EGERIA_WIDTH = int(os.environ.get("EGERIA_WIDTH", "200"))
EGERIA_ROOT_PATH = os.environ.get("EGERIA_ROOT_PATH", ".")
EGERIA_INBOX_PATH = os.environ.get("EGERIA_INBOX_PATH", ".")

console = Console(width=EGERIA_WIDTH)

debug_level = DEBUG_LEVEL
global COMMAND_DEFINITIONS

load_commands('commands.json')


log_format = "D {time} | {level} | {function} | {line} | {message} | {extra}"
logger.remove()
logger.add(sys.stderr, level="INFO", format=log_format, colorize=True)
full_file_path = os.path.join(EGERIA_ROOT_PATH, EGERIA_INBOX_PATH, "data_designer_debug.log")
# logger.add(full_file_path, rotation="1 day", retention="1 week", compression="zip", level="TRACE", format=log_format,
#            colorize=True)
logger.add("debug_log", rotation="1 day", retention="1 week", compression="zip", level="TRACE", format=log_format,
           colorize=True)
@logger.catch
def main():
    """

    """
    glossary_name = "Egeria-Markdown"
    commands = COMMAND_DEFINITIONS["Command Specifications"]
    for command, values in commands.items():
        if command == "exported":
            continue
        if commands[command].get("level","") not in ["Basic"]:
            continue
        command_description = commands[command].get("description","")
        command_verb = commands[command].get("verb","")
        print(f"\n# {command_verb} {command}\n>\t{command_description}")

        attributes = commands[command]['Attributes']
        for attribute in attributes:
            for key, value in attribute.items():
                if value.get('level',"") not in ["Basic"]:
                    continue
                user_specified =  value.get('user_specified', 'true') in ["true", "True"]

                print(f"\n## {key}")

                print(f">\tInput Required: {value.get('input_required', "false")}")

                print(f">\tDescription: {value.get("description","")}")
                labels = value.get("attr_labels", None)
                if labels:
                    print(f">\tAlternative Labels: {labels}")

                valid_values = value.get("valid_values", None)
                if valid_values:
                    print(f">\tValid Values: {valid_values}")
                default_value = value.get("default_value", None)
                if default_value:
                    print(f">\tDefault Value: {default_value}")
        print("\n___\n")




    #

if __name__ == "__main__":
    main()